program Mediana; { RTK 1990.2.2 }
{$IFDEF MERICAS}
uses Windows;
const MaxTabela = 1000000;
{$ELSE}
const MaxTabela = 10000;
{$ENDIF}
type TabelaT = array[1..MaxTabela] of real; 

function Srednji(n: integer; var a, b: TabelaT): real; 
var i, ia, ib: integer; 
begin
  ia := 1; ib := 1; 
  for i := 1 to n - 1 do 
    { Invarianta: manji izmed elementov a[ia] in b[ib] je obenem tudi 
      i-ti najmanji v uniji vseh elementov zaporedij a in b. }
    if a[ia] < b[ib] then ia := ia + 1 else ib := ib + 1; 
  if a[ia] < b[ib] then Srednji := a[ia] else Srednji := b[ib]; 
end; {Srednji} 

function Srednji2(n: integer; var a, b: TabelaT): real; 
var 
  aLevi, bLevi, aDesni, bDesni, aSrednji, bSrednji: integer; 
  Zamik: boolean; 
begin
  aLevi := 1; aDesni := n; bLevi := 1; bDesni := n; 
  while aLevi < aDesni do begin 
    aSrednji := (aLevi + aDesni) div 2; bSrednji := (bLevi + bDesni) div 2; 
    Zamik := Odd(aDesni - aLevi); 
    if a[aSrednji] <= b[bSrednji] then begin 
      aLevi := aSrednji; bDesni := bSrednji; 
      if Zamik then aLevi := aLevi + 1; 
    end else begin 
      aDesni := aSrednji; bLevi := bSrednji; 
      if Zamik then bLevi := bLevi + 1; 
    end; {if}
  end; {while}
  if a[aLevi] < b[bLevi] then Srednji2 := a[aLevi] else Srednji2 := b[bLevi]; 
end; {Srednji2} 

{-----------------------------------------------------------------------------}
{ Preostanek programa klie podprograma Srednji in/ali Srednji2 na nekaj      }
{ nakljuno sestavljenih zaporedjih.  e je pri prevajanju definiran simbol   }
{ MERICAS, bo program tudi meril as CPU, porabljen pri izvajanju             }
{ Srednji in Srednji2.  "al" pa se Srednji2 izvede tako hitro, da je njegov  }
{ as teko izmeriti.  Pri naih poskusih na procesorju PIII 800 MHz je       }
{ bila vrednost spremenljivke Cas na koncu e vedno enaka 0, torej je porabil }
{ manj kot 0.1 mikrosekunde.  Srednji je za isti poskus porabil priblino     }
{ 0.7 sekunde.                                                                }
{-----------------------------------------------------------------------------}

var n: integer; a, b: TabelaT; s: real;

{$IFDEF MERICAS}

  var Cas, CasOd: Int64; Alg: integer; Rezultat: real;
  
  function Ura: Int64;
  var ct, et, kt, ut: FILETIME; l: LARGE_INTEGER;
  begin
    { Tale funkcija iz vmesnika Win32 nam vrne med drugim as CPU, ki ga je
      doslej porabil na program.  Dobili bomo 64-bitno tevilko, 
      enota pa je 100 nanosekund. }
    GetProcessTimes(GetCurrentProcess, ct, et, kt, ut);
    l.LowPart := ut.dwLowDateTime;
    l.HighPart := ut.dwHighDateTime;
    Ura := l.QuadPart;
  end; {Ura}
  
  procedure Start;
    begin CasOd := Ura end;
  procedure Stop;
    begin Cas := Cas + (Ura - CasOd) end; 

{$ELSE}

  procedure Test;
  var s1, s2: real;
  begin
    { Vsa tri tevila bi morala biti enaka. }
    s1 := Srednji(n, a, b); s2 := Srednji2(n, a, b);
    WriteLn(s:8:0, s1:8:0, s2:8:0);
    Assert(s = s1); Assert(s = s2);
  end; {Test}

{$ENDIF}

var i, j, na, nb: integer; x: Real;
begin {Mediana}

  {$IFDEF MERICAS}
    Cas := 0; Rezultat := 0;
    if ParamCount > 0 then Val(ParamStr(1), Alg, i)
    else Alg := 2;
  {$ENDIF}
  
  n := MaxTabela; 
  { Nekaj testov z nakljunimi podatki. }
  for j := 1 to 10 do begin
    x := 0; na := 0; nb := 0;
    for i := 1 to 2 * n do begin
      x := x + (Random(5) + 1);
      if i = n then s := x;
      if (na = n) or ((nb < n) and (Random(2) = 1)) then 
        begin nb := nb + 1; b[nb] := x end
      else 
        begin na := na + 1; a[na] := x end;
    end; {for i}
    {$IFDEF MERICAS}
      Start;
      if Alg = 2 then Rezultat := Rezultat + Srednji2(n, a, b)
      else Rezultat := Rezultat + Srednji(n, a, b);
      Stop;
    {$ELSE}
      Test;
    {$ENDIF}
  end; {for j}
  
  {$IFDEF MERICAS}
    WriteLn('as: ', (Cas / 10.0):0:1, ' mikrosekund (rezultat = ', Rezultat:0:0, ')');
  {$ELSE}
    { Pa e nekaj umetnih primerov. }
    for i := 1 to n do begin a[i] := 10; b[i] := 20; end; 
    s := a[n]; Test;
    for i := 1 to n do begin a[i] := 10 + i; b[i] := 10 + n + i; end; 
    s := a[n]; Test;
    for i := 1 to n do begin a[i] := 20; b[i] := 10; end; 
    s := b[n]; Test;
    for i := 1 to n do begin a[i] := 10 + n + i; b[i] := 10 + i; end; 
    s := b[n]; Test;
    for i := 1 to n do begin a[i] := 10 + i; b[i] := 10 + i; end; 
    s := a[(n + 1) div 2]; Test;
  {$ENDIF}
  
end. {Mediana}
