program SimulatorGorilnikov; { RTK 1990.2.3 }
uses Crt;

{ Ker mora na program simulirati dogajanje na ve
  gorilnikih hkrati, smo morali podprogram Gorilnik malo
  prilagoditi.  Namesto da bi klical Cakaj, da pocaka za
  1 ms, se bo v takih primerih kar vrnil iz podprograma,
  glavni program pa ga bo ez as poklical ponovno.  Da si
  bo od enega do drugega izvajanja zapomnil svoje "lokalne
  spremenljivke", bo podprogram Gorilnik zdaj uporabljal
  strukturo tipa StanjeR. 
  
  Glavni program bo stanje stanje gorilnikov program 
  prikazoval na zaslonu.  Za vsak gorilnik imamo tri
  vrstice: v prvi je rumen napis ###, e ta gorilnik
  gori, sicer je prazno; v drugi je koliina goriva
  (temno siva, e ga je malo, sicer svetlo siva); 
  v tretji pa je temperatura (modra pri hladnih in
  rdea pri vroih gorilnikih).
  
  Simulacijo lahko zamrznemo s pritiskom na tipko P,
  s pritiskom na Q, X ali Escape pa konamo z izvajanjem
  programa.  Ko je simulacija zamrznjena, ponovni
  pritisk na P odsimulira en sam korak, pritisk na
  poljubno drugo tipko pa simulacijo spet odmrzne. 
  
  Na zaslonu naj bo vsaj 100 stolpcev in 28 vrstic, 
  da bo dovolj prostora za vse gorilnike. }

  function Netilec(St: integer): boolean; forward;
  function LeviGori(St: integer): boolean; forward;
  function DesniGori(St: integer): boolean; forward;
  
  type StanjeR = record
    Gori: boolean;     { gorilnik gori } 
    Gorivo: integer;   { koliina goriva v gorilniku } 
    Temper: integer;   { temperatura gorilnika } 
    IP: integer;       { namesto programskega tevca }
  end; {if}

procedure Gorilnik(St: integer; var Stanje: StanjeR);
begin {Gorilnik} 
  with Stanje do begin
  
    if IP = 0 then begin
      Gori := false; Gorivo := 100; Temper := 0; 
      if Netilec(St) then { natanko en procesor je "netilec"}
        begin Gori := true; Temper := 100 end; 
      IP := 1; exit;
    end; {if}
    
    if IP = 1 then begin
      if (Temper < 80) or (Gorivo < 50) then begin { ne gori } 
        if LeviGori(St) or DesniGori(St) then begin { segrevanje od sosedov } 
          Temper := Temper + 11; 
          if Temper > 100 then Temper := 100 
        end; {if}
        if Temper > 0 then Temper := Temper - 1;    { ohlajanje } 
        if Gorivo < 100 then Gorivo := Gorivo + 1;  { dotok goriva } 
        exit;
      end; {if}
      IP := 2;
      Gori := true; Temper := 100;                  { vig } 
    end; {if}
    
    if IP = 2 then begin
      if Gorivo >= 10 then                          { gorenje } 
        begin Gorivo := Gorivo - 10; exit end;
      IP := 1; 
      Gori := false; Gorivo := 0;                   { ugasnitev } 
    end; {if}
    
  end; {with}
end; {Gorilnik} 

const StGorilnikov = 100;
var Stanje, StaroStanje: array[1..StGorilnikov] of StanjeR;
    Cas: integer;

function Netilec(St: integer): boolean;
begin
  Netilec := St = 48;
end; {Netilec}

function LeviGori(St: integer): boolean; 
begin
  if St = 1 then St := StGorilnikov else St := St - 1;
  LeviGori := StaroStanje[St].Gori;
end; {LeviGori}

function DesniGori(St: integer): boolean; 
begin
  if St = StGorilnikov then St := 1 else St := St + 1;
  DesniGori := StaroStanje[St].Gori;
end; {DesniGori}

procedure NarisiGorilnike;
var i, x, y: integer;
begin
  for i := 1 to StGorilnikov do begin
    if  (Stanje[i].Gori = StaroStanje[i].Gori)
    and (Stanje[i].Gorivo = StaroStanje[i].Gorivo)
    and (Stanje[i].Temper = StaroStanje[i].Temper)
    then continue;
    { Doloimo koordinate gorilnika na zaslonu. 
      Potrebovali bomo 100 stolpcev in 28 vrstic. }
    case i of
    1..24: begin x := (i - 1) * 4 + 1; y := 1; end;
    25: begin x := 93; y := 5; end;
    26..47: begin x := (49 - i) * 4 + 1; y := 9; end;
    48: begin x := 9; y := 13; end;
    49..70: begin x := (i - 47) * 4 + 1; y := 17; end;
    71: begin x := 93; y := 21; end;
    72..95: begin x := (95 - i) * 4 + 1; y := 25; end;
    96..100: begin x := 1; y := (101 - i) * 4 + 1; end;
    end; {case}
    StaroStanje[i] := Stanje[i];
    with Stanje[i] do begin
      GotoXY(x, y); TextColor(Yellow); if Gori then Write('###') else Write('   ');
      GotoXY(x, y + 1); 
      if Gorivo <= 50 then TextColor(DarkGray) else TextColor(LightGray);
      Write(Gorivo:3);
      if Temper <= 33 then TextColor(Blue) 
      else if Temper <= 66 then TextColor(LightBlue)
      else TextColor(LightRed);
      GotoXY(x, y + 2); Write(Temper:3);
      TextColor(LightGray);
    end; {with}
  end; {for i}
  GotoXY(1, 28); Write('T = ', (Cas * 0.1):0:1, ' s');
end; {NarisiGorilnike}

procedure Korak;
var i: integer;
begin
  for i := 1 to StGorilnikov do Gorilnik(i, Stanje[i]);
  NarisiGorilnike;
  Cas := Cas + 1;
end; {Korak}

var i: integer; ch: char; VTeku: boolean;
begin {SimulatorGorilnikov}
  ClrScr; 
  CursorOff;
  for i := 1 to StGorilnikov do begin 
    StaroStanje[i].Gorivo := -1;
    Stanje[i].IP := 0; Gorilnik(i, Stanje[i]);
  end; {for i}
  VTeku := true; Cas := 0;
  while true do begin
    if VTeku then Korak;
    if VTeku and not KeyPressed then begin Delay(50); continue; end;
    ch := UpCase(ReadKey);
    if ch = #0 then ReadKey; 
    if ch = 'P' then begin
      if VTeku then VTeku := false
      else Korak;
    end else if (ch = 'Q') or (ch = 'X') or (ch = #27) then break
    else if not VTeku then VTeku := true;
  end; {while}
  CursorOn;
end. {SimulatorGorilnikov}
