program ZelezniskaPostaja; { RTK 1990.3.2 }

{ Program z animacijo na zaslonu prikazuje delovanje treh
  algoritmov za urejanje vagonov.  S pritiskom na tipko P
  lahko zamrznemo animacijo; potem s ponovnim pritiskom
  na P simuliramo samo naslednji korak, s pritiskom na
  poljubno drugo tipko pa odmrznemo animacijo.  S pritiskom
  na Q, X ali Escape prekinemo izvajanje programa. 
  tevilko algoritma (1, 2 ali 3) podamo programu kot prvi
  parameter v ukazni vrstici. }

uses Crt;
const 
  n = 5;                { tevilo slepih tirov } 
  VhodniTir = 0;        { indeks vhodnega tira } 
  IzhodniTir = n + 1;   { indeks izhodnega tira } 

  procedure PremakniVagon(OdKod, Kam: integer); forward;
  function Prazen(Tir: integer): boolean; forward;
  function Vagon(Tir: integer): integer; forward;

{-----------------------------------------------------------------------------}

{ Zlije vsebino tirov 1..StTirov na tir StTirov + 1.  V zlivanje vkljui 
  tudi prvi vagon s tira 0.  e je Obrnjeno = true, predpostavi, da so 
  vagoni na vhodnih tirih urejeni padajoe.  Na tiru StTirov + 1 bo 
  urejenost v vsakem primeru ravno nasprotna. }
procedure Zlivanje(StTirov: integer; Obrnjeno: boolean); 
var Ze0: boolean; Tir, Min, KjeMin: integer; 
begin
  Ze0 := false; { Vagona s tira 0 e nismo premaknili. } 
  repeat 
    { Poiimo med prvimi vagoni s tirov 1..StTirov najmanjega 
      (ali najvejega, e je Obrnjeno = true).  e je Ze0 = false, 
      gledamo tudi tir 0. }
    KjeMin := -1; if Ze0 then Tir := 1 else Tir := 0; 
    while Tir <= StTirov do begin 
      if not Prazen(Tir) then 
        if (KjeMin < 0) or ((Vagon(Tir) < Min) <> Obrnjeno) then 
          begin Min := Vagon(Tir); KjeMin := Tir; end;
      Tir := Tir + 1; 
    end; {while}
    { Premaknimo najdeni vagon na tir StTirov + 1. } 
    if KjeMin >= 0 then PremakniVagon(KjeMin, StTirov + 1); 
    if KjeMin = 0 then Ze0 := true; { s tira 0 ne smemo ve brati } 
  until KjeMin < 0; 
end; {Zlivanje} 

{ Predpostavi, da so tiri 1..StTirov trenutno prazni.  Poskrbi, da se za vse 
  i od 1 do StTirov na tiru i nahaja 1 shl i vagonov, urejenih naraajoe, 
  e je Obrnjeno = true, in padajoe, e je Obrnjeno = false.  Vse te vagone
  vzame z vhodnega tira 0. }
procedure NapolniTire(StTirov: integer; Obrnjeno: boolean); 
begin
  if StTirov = 1 then PremakniVagon(0, 1) 
  else begin 
    { Napolnimo prvih StTirov - 1 tirov v nasprotnem vrstnem redu. } 
    NapolniTire(StTirov - 1, not Obrnjeno); 
    { Z zlivanjem dobimo na tiru StTirov ravno prav vagonov 
      v ravno pravem vrstnem redu. }
    Zlivanje(StTirov - 1, not Obrnjeno); 
    { Prvih StTirov - 1 je spet praznih, napolnimo jih zdaj v 
      elenem vrstnem redu. }
    NapolniTire(StTirov - 1, Obrnjeno); 
  end; {if}
end; {NapolniTire} 

procedure ZelezniskaPostaja1;
begin 
  { Pripravimo na i-tem tiru 1 shl i vagonov v pravem vrstnem redu, 
    za vse i od 1 do n.  En vagon ostane e na vhodnem tiru. }
  NapolniTire(n, false); 
  { Zdaj jih zlijemo na izhodni tir.  Kot si elimo, bodo 
    prili vagoni z nijimi tevilkami prej na izhodni tir. }
  Zlivanje(n, false); 
end; {ZelezniskaPostaja1} 

{-----------------------------------------------------------------------------}

{ Ta podprogram opravi en korak urejanja za vagone s tevilkami StOd..StDo.  
  Trenutno je treba nekaj narediti z vagonom StVagona, ki prihaja s tira 
  STira. e je ta vagon s prve polovice intervala StOd..StDo, ga bomo predali 
  podprogramu za urejanje te polovice (z rekurzivnim klicem), sicer pa ga 
  bomo odloili na pomoni tir. }
procedure PrenosNaprej(STira, PomozniTir, StOd, StDo, StVagona: integer); 
var Meja: integer; 
begin
  Meja := (StOd + StDo) div 2; 
  { Vagone (Meja + 1)..StDo bomo odloili na pomoni tir,
    vagone StOd..Meja pa bomo prenesli naprej.  Pri StOd = StDo 
    je moen itak en sam vagon in pomoni tir je tedaj isti kot izhodni. }
  if (StVagona > Meja) or (StOd = StDo) then PremakniVagon(STira, PomozniTir) 
  else PrenosNaprej(STira, PomozniTir + 1, StOd, Meja, StVagona); 
end; {PrenosNaprej} 

{ Predpostavi, da se na tiru StTira nahajajo vagoni StOd..StDo.  Poskrbi, 
  da se ta tir sprazni in vsi vagoni pridejo na izhodni tir v pravem vrstnem redu. }
procedure SprazniTir(StTira, StOd, StDo: integer); 
var Meja: integer; 
begin
  Meja := (StOd + StDo) div 2; 
  if StTira < n then if not Prazen(StTira + 1) then 
    { Naslednji tir bomo potrebovali kot pomoni tir, a so na njem 
      e stari vagoni, torej najprej spraznimo tega. }
    SprazniTir(StTira + 1, StOd - (StDo - Meja), StOd - 1); 
  { Spraznimo zdaj zahtevani tir. } 
  while not Prazen(StTira) do begin 
    PrenosNaprej(StTira, StTira + 1, StOd, StDo, Vagon(StTira)); 
  end; {while}
  { Z rekurzivnim klicem spraznimo e naslednje pomone tire. } 
  if StTira < n then SprazniTir(StTira + 1, Meja + 1, StDo); 
end; {SprazniTir} 

procedure ZelezniskaPostaja2;
begin 
  SprazniTir(VhodniTir, 1, 1 shl n); 
end; {ZelezniskaPostaja2} 

{-----------------------------------------------------------------------------}

{ Vrne indeks najvijega priganega bita v x med biti 0..(m - 1).   
  e so same nile, vrne -1. }
function NajvisjiBit(x, m: integer): integer; 
var j: integer; 
begin
  j := m - 1;  
  while j >= 0 do 
    if (x and (1 shl j)) <> 0 then break
    else j := j - 1; 
  NajvisjiBit := j; 
end; {NajvisjiBit} 

procedure Uredi(Tir: integer); 
var i, StBitov: integer; 
begin
  { Predpostavka: na tiru Tir so vagoni, katerih tevilke (e bi jih teli od  
    0 naprej namesto od 1 naprej) bi se razlikovale le v spodnjih StBitov 
    bitih, tiri od Tir + 1 do n pa so prazni. }
  StBitov := n - Tir; 
  { Razporedimo vagone med tire Tir + 1 do n + 1 glede na najviji prigani 
    bit med spodnjimi StBitov biti.  To nam zagotovi, da pridejo vsi s 
    tira i + 1 v vrstnem redu pred vsemi s tira i. }
  while not Prazen(Tir) do 
    PremakniVagon(Tir, n - NajvisjiBit(Vagon(Tir) - 1, StBitov)); 
  { Poistimo tire od n do Tir + 1. } 
  for i := n downto Tir + 1 do Uredi(i); 
  { Rezultat: prazni so tiri od Tir do n, vagone, ki so bili prej na tiru 
    Tir, pa smo zdaj v pravem vrstnem redu premaknili na izhodni tir. }
end; {Uredi} 

procedure ZelezniskaPostaja3;
begin {ZelezniskaPostaja3} 
  Uredi(0); 
end; {ZelezniskaPostaja3} 

{-----------------------------------------------------------------------------}

const StVagonov = 1 shl n;
type TirT = array[1..StVagonov] of integer;
var Tiri: array[VhodniTir..IzhodniTir] of TirT;
    StNaTiru: array[VhodniTir..IzhodniTir] of integer;

function Prazen(Tir: integer): boolean; 
begin
  Prazen := StNaTiru[Tir] = 0;
end; {Prazen}

function Vagon(Tir: integer): integer; 
begin
  Assert(StNaTiru[Tir] > 0);
  Vagon := Tiri[Tir, StNaTiru[Tir]];
end; {Vagon}

{ Narie vsebino tira, zamaknjeno za Zamik znakov v levo.
  Na zaetku vrstice napie tudi tevilko tira (V = vhodni, I = izhodni). }
procedure NarisiTir(Tir, Zamik: integer);
var i: integer;
begin
  GotoXY(1, Tir * 2 + 1); 
  if Tir = VhodniTir then Write('V') 
  else if Tir = IzhodniTir then Write('I')
  else Write(Tir);
  for i := 1 to (StVagonov - StNaTiru[Tir]) * 3 - Zamik do Write(' ');
  for i := 1 to StNaTiru[Tir] do Write(Tiri[Tir, i]:3);
  if Zamik > 0 then for i := 1 to Zamik do Write(' ');
end; {NarisiTir}

var VTeku: boolean; StPremikov: integer;

procedure Pocakaj;
var c: char;
begin
  while true do begin
    if KeyPressed or not VTeku then begin
      c := UpCase(ReadKey);
      if c = #0 then ReadKey;
      if c = 'P' then begin
        if VTeku then VTeku := false
        else exit;
      end else if (c = 'Q') or (c = 'X') or (c = #27) then Halt(0)
      else if VTeku then VTeku := false;
    end; {if}
    if VTeku then 
      begin Delay(10); exit end;
  end; {while}
end; {Pocakaj}

{ Premakne vagon z enega tira na drugega.  Na zaslonu to
  prikae s preprosto animacijo. }
procedure PremakniVagon(OdKod, Kam: integer); 
const DolzinaTira = 3 * (StVagonov - 1) + 1;
var V, i, y, yKam: integer;
begin
  Assert(StNaTiru[OdKod] > 0);
  Assert(StNaTiru[Kam] < n);
  if KeyPressed then Halt;
  V := Tiri[OdKod, StNaTiru[OdKod]]; y := 2 * OdKod + 1;
  StNaTiru[OdKod] := StNaTiru[OdKod] - 1;
  for i := 1 to 6 do begin
    NarisiTir(OdKod, 3 - (i div 2));
    GotoXY(DolzinaTira + i, y); Write(V:3);
    Pocakaj;
  end; {for i}
  yKam := 2 * Kam + 1;
  while y <> yKam do begin
    GotoXY(DolzinaTira + 6, y); Write('   ');
    if y < yKam then y := y + 1 else y := y - 1;
    GotoXY(DolzinaTira + 6, y); Write(V:3);
    Pocakaj;
  end; {while}
  for i := 5 downto 1 do begin
    NarisiTir(Kam, 3 - (i div 2));
    GotoXY(DolzinaTira + i, y); Write(V:3, ' ');
    Pocakaj;
  end; {for i}
  StNaTiru[Kam] := StNaTiru[Kam] + 1;
  Tiri[Kam, StNaTiru[Kam]] := V;
  NarisiTir(Kam, 0);
  StPremikov := StPremikov + 1;
  GotoXY(1, 2 * n + 5); Write('tevilo premikov: ', StPremikov);
end; {PremakniVagon}

var Alg, i, j: integer;
begin {ZelezniskaPostaja}

  { Na vhodnem tiru naj bodo nakljuno premeani vagoni.
    Ostali tiri naj bodo prazni. }
  Randomize;
  for i := 1 to IzhodniTir do StNaTiru[i] := 0;
  StNaTiru[VhodniTir] := StVagonov;
  for i := 1 to StVagonov do begin
    j := Random(i) + 1;
    Tiri[VhodniTir, i] := Tiri[VhodniTir, j];
    Tiri[VhodniTir, j] := i;
  end; {for}

  { Kateri algoritem bomo uporabili? }
  if ParamCount > 0 then Val(ParamStr(1), Alg, i)
  else Alg := 1;
  
  ClrScr; VTeku := True; StPremikov := 0;
  for i := VhodniTir to IzhodniTir do NarisiTir(i, 0);
  case Alg of
  1: ZelezniskaPostaja1;
  2: ZelezniskaPostaja2;
  3: ZelezniskaPostaja3;
  end; {Alg}
  
end. {ZelezniskaPostaja}
