program Izpis(Input, Output); { RTK 1991.1.3 }
const MaxDolz = 10;
type 
  PisavaT = (Normalno, Nagnjeno, Krepko); 
  BesedaT = packed array[1..MaxDolz] of char; 

{-----------------------------------------------------------------------------}
{ Podprogrami, za katere naloga predpostavi, da so e podani.                 }
{ Olepano izvorno kodo izpisujejo na standardni izhod v obliki HTML.         }
{-----------------------------------------------------------------------------}
  
procedure PreberiZnak(var c: char); 
begin 
  Read(c); 
end; {PreberiZnak}

procedure IzpisiZnak(c: char); 
begin
  case c of
    '&': Write('&amp;');
    '"': Write('&quot;');
    '<': Write('&lt;');
    '>': Write('&gt;');
    else Write(c);
  end; {case}
end; {IzpisiZnak}

var TrenPisava: PisavaT;

procedure ZacniTrenutnoPisavo;
begin
  if TrenPisava = Nagnjeno then Write('<i>')
  else if TrenPisava = Krepko then Write('<b>');
end; {ZacniTrenutnoPisavo}

procedure KoncajTrenutnoPisavo;
begin
  if TrenPisava = Nagnjeno then Write('</i>')
  else if TrenPisava = Krepko then Write('</b>');
end; {KoncajTrenutnoPisavo}

procedure SpremeniPisavo(Pisava: PisavaT); 
begin
  if Pisava = TrenPisava then exit;
  KoncajTrenutnoPisavo;
  TrenPisava := Pisava;
  ZacniTrenutnoPisavo;
end; {SpremeniPisavo}

function RezervBeseda(b: BesedaT; bL: integer): boolean; 
const 
  StBesed = 35;
  { V ISO-standardu za Pascal nisem nael izraza "reserved word".
    Tole spodaj so "word-symbols" (razdelek 6.1.2 v standardu). }
  Besede: array[1..StBesed] of string = (
    'and', 'array', 'begin', 'case', 'const', 'div',
    'do', 'downto', 'else', 'end', 'file', 'for',
    'function', 'goto', 'if', 'in', 'label', 'mod',
    'nil', 'not', 'of', 'or', 'packed', 'procedure',
    'program', 'record', 'repeat', 'set', 'then',
    'to', 'type', 'until', 'var', 'while', 'with');
var i, j: integer;
begin
  RezervBeseda := true;
  for i := 1 to StBesed do if Length(Besede[i]) = bL then begin
    j := 1; while j <= bL do 
      if b[j] = Besede[i, j] then j := j + 1 else break;
    if j > bL then exit;
  end; {for}
  RezervBeseda := false;
end; {RezervBeseda}

procedure ZacetekIzpisa;
begin
  TrenPisava := Normalno;
  WriteLn('<html><head><title>Izvorna koda</title></head><body>');
  Write('<pre>');
end; {ZacetekIzpisa}

procedure KonecIzpisa;
begin
  KoncajTrenutnoPisavo;
  WriteLn('</pre></body></html>');
end; {KonecIzpisa}

{-----------------------------------------------------------------------------}
{ Spodaj je tisti del programa, ki naj bi ga napisal tekmovalec.              }
{-----------------------------------------------------------------------------}

var 
  c: char; 
  Beseda: BesedaT; 
  BesedaL: integer; 
  i: integer; 
  Niz, Komentar, Rez: boolean; 
  
  procedure KonecBesede;
  begin
    { Prili smo do konca besede.  e je to zelo dolga beseda, smo jo
      izpisovali e sproti in je zdaj ni treba, sicer pa bo treba 
      preveriti, e to ni sluajno ena od rezerviranih besed. }
    if BesedaL <= MaxDolz then begin
      Rez := RezervBeseda(Beseda, BesedaL); 
      if Rez then SpremeniPisavo(Krepko); 
      for i := 1 to BesedaL do IzpisiZnak(Beseda[i]); 
      if Rez then SpremeniPisavo(Normalno); 
    end; {if}
    BesedaL := 0; 
  end; {KonecBesede}
  
begin {Izpis} 
  ZacetekIzpisa;

  BesedaL := 0; Niz := false; Komentar := false; SpremeniPisavo(Normalno); 
  while not Eof do begin 
    PreberiZnak(c); 
    if (BesedaL > 0) and not (c in ['A'..'Z', 'a'..'z', '0'..'9']) then 
      KonecBesede;
    if Niz then begin 
        if c = '''' then Niz := false; IzpisiZnak(c); 
    end else if Komentar then begin 
      IzpisiZnak(c); 
      if c = '}' then 
        begin Komentar := false; SpremeniPisavo(Normalno); end; 
    end else if c = '{' then begin 
      Komentar := true; SpremeniPisavo(Nagnjeno); IzpisiZnak(c); 
    end else if c = '''' then begin 
      IzpisiZnak(c); Niz := true; 
    end else if c in ['A'..'Z', 'a'..'z', '0'..'9'] then begin 
      if BesedaL = MaxDolz then begin
        { Trenutna beseda bo dolga vsaj MaxDolz + 1, torej to ni
          rezervirana beseda.  Izpiimo, kar smo e shranili,
          ostalo pa bomo izpisovali sproti.  BesedaL bo ostala
          pri vrednosti MaxDolz + 1. }
        for i := 1 to BesedaL do IzpisiZnak(Beseda[i]);
        BesedaL := BesedaL + 1; 
      end; {if}
      if BesedaL > MaxDolz then IzpisiZnak(c)
      else begin BesedaL := BesedaL + 1; Beseda[BesedaL] := c; end;
    end else IzpisiZnak(c);  
  end; {while}
  if BesedaL > 0 then KonecBesede;
  
  KonecIzpisa;
end. {Izpis}
