program StiskanjeVrstic; { RTK 1991.2.2 }
const VrstaM = 999; 
type 
  SkatlaT = record Visina, Globina, Sirina: integer end;
  VrstaT = array[1..VrstaM] of SkatlaT; 

procedure Narisi(var a: VrstaT; aL: integer;
                 var b: VrstaT; bL: integer;
                 ZamikX, RazmikY: integer); forward;

{-----------------------------------------------------------------------------}
{ Spodnji podprogram je reitev, kakrno je zahtevala prvotna naloga.         }
{-----------------------------------------------------------------------------}

{ Funkcija vrne najmanjo razdaljo med sosednjima vrstama pravokotnikov, 
  pri kateri se pravokotniki obeh vrstic e ne prekrivajo med seboj. }
function NajmanjsaRazdalja(var a: VrstaT; aL: integer; 
                           var b: VrstaT; bL: integer): integer; 
var 
  Konec: boolean; 
  ia, ib: 1..VrstaM; { indeksa trenutno pregledovanega 
                       pravokotnika v obeh vrsticah } 
  SirinaA, SirinaB: integer; { doslej pregledani dolini obeh vrstic } 
  StaraSirinaA: integer; 
  Razdalja: integer; { globina gornjega pravokotnika 
                       + viina spodnjega v trenutni toki } 
  DosedanjaRazdalja: integer; { najveja razdalja doslej } 
begin
  if (aL = 0) or (bL = 0) then begin NajmanjsaRazdalja := 0; exit end;
  ia := 1; ib := 1; Konec := false; 
  SirinaA := a[ia].Sirina; SirinaB := b[ib].Sirina; 
  DosedanjaRazdalja := a[ia].Globina + b[ib].Visina; 
  repeat 
    StaraSirinaA := SirinaA; 
    if SirinaA <= SirinaB then begin 
      if ia >= aL then Konec := true
      else begin ia := ia + 1; SirinaA := SirinaA + a[ia].Sirina; end;
    end; {if}
    if SirinaB <= StaraSirinaA then begin 
      if iB >= bL then Konec := true
      else begin ib := ib + 1; SirinaB := SirinaB + b[ib].Sirina; end;
    end; {if}
    if not Konec then begin 
      Razdalja := a[ia].Globina + b[ib].Visina; 
      if Razdalja > DosedanjaRazdalja then 
        DosedanjaRazdalja := Razdalja; 
    end; {if}
  until Konec; 
  NajmanjsaRazdalja := DosedanjaRazdalja; 
end; {NajmanjsaRazdalja}

{-----------------------------------------------------------------------------}
{ Reitev, ki dopua tudi premikanje spodnje vrstice levo ali desno, e      }
{ lahko s tem kaj zmanjamo razmik med osnovnicama.                           }
{-----------------------------------------------------------------------------}

{ Rezultat vrne prek parametrov ZamikX in RazmikY. }
procedure NajmanjsaRazdalja2(var a: VrstaT; aL: integer; 
                             var b: VrstaT; bL: integer;
                             var ZamikX, RazmikY: integer); 
{ Do prvega stika pride pri tistem x, ko je prvi prisoten po
  en pravokotnik z vsake vrstice.  Dotlej smo torej morali priti
  mimo vsaj dveh levih stranic pravokotnikov.  Od tam naprej pa
  vsaki, ko se zane nek nov pravokotnik, nastopi tudi nov stik.
  Vsega skupaj je torej lahko stikov najve aL + bL - 1. }
const MaxStikov = 2 * VrstaM - 1;
type
  StikT = record
    ia, ib: integer;
  end; {StikT}
  { Tabela za kopice in kazala. }
  TabelaT = array[1..MaxStikov] of integer;
var 
  { Kopici vsebujeta indekse stikov (1..StStikov).  
    Urejeni sta tako, da so stiki z manjo vrednostjo pri vrhu.
    Ker bi pri tabeli KopicaR v resnici hoteli vedeti najveji
    razmik, bomo razmike tam pomnoili z -1.
    Za kazali pa velja: Kopica[Kazalo[i]] = i. }
  KopicaR, KopicaN, KazaloR, KazaloN: TabelaT;
  Stiki: array[1..MaxStikov] of StikT; StStikov: integer;
  StikiR: TabelaT; { za vsak stik hrani razmik med osnovnicama, 
    ki ga zahteva ta stik; torej R = a[ia].Globina + b[ib].Visina }
  StikiN: TabelaT; { za vsak stik hrani naslednji t, pri katerem 
                     bo ta stik zahteval kakno spremembo }
  t: integer; { zamik spodnje vrstice glede na zgornjo }
  { StikiD so tisti, ki jih bo treba pred prehodom na veji t e dodati. }
  StikiD: array[1..MaxStikov] of StikT; StStikovD: integer;
  LeviRobA, LeviRobB: array[1..VrstaM + 1] of integer;
  SirinaA, SirinaB: integer; { skupna irina posamezne vrstice }
  
  { Izrauna naslednjo vrednost t, pri kateri bo stik (ia, ib)
    povzroil kakno spremembo. }
  function IzracunajN(ia, ib: integer): integer;
  var a1, a2, b1, b2, N: integer;
  begin
    { Zgornji pravokotnik stika pokriva x-koordinate 
      z intervala [a1, a2], spodnji pa [b1, b2]. }
    a1 := LeviRobA[ia]; a2 := a1 + a[ia].Sirina;
    b1 := t + LeviRobB[ib]; b2 := b1 + b[ib].Sirina;
    Assert(b1 < a2); Assert(b2 >= a1);
    { Ko se b1 dotakne a2, bo ta stik izginil. }
    N := t + a2 - b1;
    { Ko se b2 dotakne a2, bo nastal nov stik (med a[ia + 1] in b[ib]). }
    if (b2 < a2) and (ia < aL) then 
      if t + a2 - b2 < N then N := t + a2 - b2;
    { Ko se b1 dotakne a1, bo nastal nov stik (med a[ia] in b[ib - 1]). }
    if (b1 < a1) and (ia = 1) and (ib > 1) then
      if t + a1 - b1 < N then N := t + a1 - b1;
    IzracunajN := N;
    {WriteLn('IzracunajN(', ia, '[', a1, ', ', a2, '], ', ib, '[', b1, ', ', b2, '], t = ', t, ') = ', N);}
  end; {IzracunajN}

  procedure IzpisiKopico(var Kopica, Kazalo, Kljuci: TabelaT);
  var i: integer;
  begin
    Write('Kopica:');
    for i := 1 to StStikov do 
      Write(' (', Stiki[Kopica[i]].ia, ', ', Stiki[Kopica[i]].ib, '):', Kljuci[Kopica[i]]);
    WriteLn;
  end; {IzpisiKopico}

  { Dviguje dani stik po kopici, dokler ne pride tik 
    pod takega, ki ima manji klju od njega. }
  procedure DvigniVKopici(var Kopica, Kazalo, Kljuci: TabelaT; StStika: integer);
  var i, pi, Kljuc: integer;
  begin
    {WriteLn('DvigniVKopici(', Stiki[StStika].ia, ', ', Stiki[StStika].ib, ')');}
    i := Kazalo[StStika]; Kljuc := Kljuci[StStika];
    while i > 1 do begin
      pi := i div 2;
      if Kljuci[Kopica[pi]] <= Kljuc then break;
      Kazalo[Kopica[pi]] := i; Kopica[i] := Kopica[pi]; i := pi;
    end; {while}
    Kazalo[StStika] := i; Kopica[i] := StStika;
    {IzpisiKopico(Kopica, Kazalo, Kljuci);}
  end; {DvigniVKopici}

  { Pogreza dani stik po kopici, dokler ne pride tik 
    pod takega, ki ima manji klju od njega. }
  procedure PogrezniVKopici(var Kopica, Kazalo, Kljuci: TabelaT; StStika: integer);
  var i, ci, Kljuc: integer;
  begin
    {WriteLn('PogrezniVKopici(', Stiki[StStika].ia, ', ', Stiki[StStika].ib, ')');}
    i := Kazalo[StStika]; Kljuc := Kljuci[StStika];
    while i <= StStikov div 2 do begin
      ci := 2 * i;
      if ci < StStikov then 
        if Kljuci[Kopica[ci + 1]] < Kljuci[Kopica[ci]] then ci := ci + 1;
      if Kljuc <= Kljuci[Kopica[ci]] then break;
      Kazalo[Kopica[ci]] := i; Kopica[i] := Kopica[ci]; i := ci;
    end; {while}
    Kazalo[StStika] := i; Kopica[i] := StStika;
    {IzpisiKopico(Kopica, Kazalo, Kljuci);}
  end; {PogrezniVKopici}

  { Doda nov stik v tabele Stiki, StikiR in StikiN ter v obe kopici. }
  procedure DodajStik(ia, ib: integer);
  begin
    {WriteLn('DodajStik(', ia, ', ', ib, ')');}
    Inc(StStikov);
    Stiki[StStikov].ia := ia; Stiki[StStikov].ib := ib;
    StikiR[StStikov] := -(a[ia].Globina + b[ib].Visina);
    StikiN[StStikov] := IzracunajN(ia, ib);
    KazaloR[StStikov] := StStikov; KopicaR[StStikov] := StStikov; 
    DvigniVKopici(KopicaR, KazaloR, StikiR, StStikov);
    KazaloN[StStikov] := StStikov; KopicaN[StStikov] := StStikov; 
    DvigniVKopici(KopicaN, KazaloN, StikiN, StStikov);
  end; {DodajStik}

  procedure DodajStikD(ia, ib: integer);
  begin
    Inc(StStikovD);
    StikiD[StStikovD].ia := ia; StikiD[StStikovD].ib := ib;
  end; {DodajStikD}

  procedure SpremembeNaStiku(StStika, ia, ib: integer);
  var a1, a2, b1, b2, ZadnjiVR, ZadnjiVN: integer;
  begin
    Assert(StikiN[StStika] = t);
    { Zgornji pravokotnik stika pokriva x-koordinate 
      z intervala [a1, a2], spodnji pa [b1, b2]. }
    a1 := LeviRobA[ia]; a2 := a1 + a[ia].Sirina;
    b1 := t + LeviRobB[ib]; b2 := b1 + b[ib].Sirina;
    {WriteLn('t = ', t, ', sprememba na stiku (', ia, '[', a1, ', ', a2, '], ', ib, '[', b1, ', ', b2, '])');}
    { Mogoe nastane zdaj kak nov stik. }
    if (b2 = a2) and (ia < aL) then DodajStikD(ia + 1, ib);
    if (b1 = a1) and (ia = 1) and (ib > 1) then DodajStikD(ia, ib - 1);
    { Mogoe bo treba trenutni stik pobrisati. }
    if b1 = a2 then begin
      { Najprej ga pobriimo iz kopic.  Na njegovo mesto
        v vsaki kopici vpiimo zadnji element tiste kopice, nato 
        pa bo treba tega mogoe e kaj pogrezniti ali dvigniti. }
      {WriteLn('Briem stik (', ia, ', ', ib, ')');}
      ZadnjiVR := KopicaR[StStikov]; ZadnjiVN := KopicaN[StStikov];
      KopicaR[KazaloR[StStika]] := ZadnjiVR;
      KazaloR[ZadnjiVR] := KazaloR[StStika];
      KopicaN[KazaloN[StStika]] := ZadnjiVN;
      KazaloN[ZadnjiVN] := KazaloN[StStika];
      Dec(StStikov);
      DvigniVKopici(KopicaR, KazaloR, StikiR, ZadnjiVR);
      PogrezniVKopici(KopicaR, KazaloR, StikiR, ZadnjiVR);
      DvigniVKopici(KopicaN, KazaloN, StikiN, ZadnjiVN);
      PogrezniVKopici(KopicaN, KazaloN, StikiN, ZadnjiVN);
      { Nato pa e premaknimo doslej zadnji stik na indeks StStika. 
        e je bil StStika sam zadnji, ni treba storiti niesar.
        Po kopicah e prav posebej ne smemo ariti, ker tega stika v njiju 
        ni ve in je Kazalo*[StStikov + 1] v bistvu neveljavna vrednost. }
      if StStika < StStikov + 1 then begin
        Stiki[StStika] := Stiki[StStikov + 1];
        StikiR[StStika] := StikiR[StStikov + 1]; StikiN[StStika] := StikiN[StStikov + 1];
        KazaloR[StStika] := KazaloR[StStikov + 1]; KazaloN[StStika] := KazaloN[StStikov + 1];
        KopicaR[KazaloR[StStika]] := StStika; KopicaN[KazaloN[StStika]] := StStika;
      end; {if}
      {Write('KopicaR: '); IzpisiKopico(KopicaR, KazaloR, StikiR);
      Write('KopicaN: '); IzpisiKopico(KopicaN, KazaloN, StikiN);}
    end
    { e pa ga ne pobriemo, mu vsaj izraunajmo novi N. }
    else begin
      StikiN[StStika] := IzracunajN(ia, ib);
      Assert(StikiN[StStika] > t);
      PogrezniVKopici(KopicaN, KazaloN, StikiN, StStika);
    end; {if}
  end; {SpremembeNaStiku}

  procedure ShraniResitevCeJeBoljsa;
  var TrenRazmikY: integer;
  begin
    Assert(StStikov > 0);
    { Najveji razmik, ki ga zahtevajo trenutno prisotni stiki,
      je zapisan v korenskem elementu kopice. }
    TrenRazmikY := -StikiR[KopicaR[1]];
    if TrenRazmikY < RazmikY then begin
      { Nali smo novo najboljo reitev. }
      RazmikY := TrenRazmikY; ZamikX := t; 
    end; {if}
    WriteLn('ZamikX: ', t, ', RazmikY: ', TrenRazmikY);
    Narisi(a, aL, b, bL, t, TrenRazmikY);
    WriteLn;
  end; {ShraniResitevCeJeBoljsa}

var i, j, tNovi: integer;
begin {NajmanjsaRazdalja2}
  if (aL = 0) or (bL = 0) then exit;
  LeviRobA[1] := 0; for i := 1 to aL do LeviRobA[i + 1] := LeviRobA[i] + a[i].Sirina;
  LeviRobB[1] := 0; for i := 1 to bL do LeviRobB[i + 1] := LeviRobB[i] + b[i].Sirina;
  SirinaA := LeviRobA[aL + 1]; SirinaB := LeviRobB[bL + 1];
  { Za zaetek imejmo le stik prvega pravokotnika zgornje in
    zadnjega pravokotnika spodnje vrstice. }
  t := -SirinaB; StStikov := 0; StStikovD := 0; DodajStik(1, bL);
  ZamikX := t; RazmikY := -StikiR[1]; { zaetni kandidat za reitev }
  { Zdaj pa poasi zamikajmo spodnjo vrstico proti desni. }
  while StStikov > 0 do begin
    i := KopicaN[1]; tNovi := StikiN[i];
    if tNovi > t then begin
      { Naslednja sprememba nastopi pri tNovi, ki je veji od
        trenutnega t.  Torej smo pri trenutnem t e pobrisali
        vse stike, ki se ravno pri tem t prekinejo.  Zdaj lahko
        pogledamo minimalni potrebni razmik med osnovnicama
        (pri tem t), saj je to kandidat za najboljo mono reitev. }
      ShraniResitevCeJeBoljsa;
      { Med pregledovanjem stikov, pri katerih je pri trenutnem t
        nastopila kakna sprememba, smo mogoe opazili nekaj stikov,
        ki nastanejo pri t in veljajo, e je zamik spodnje vrstice
        veji od t.  Te nove stike smo zaasno odlagali v tabelo
        StikiD, zdaj pa jih lahko dodamo v tabelo Stiki.  (e bi jih 
        e sproti dodajali v tabelo Stiki, bi spregledali "tesne"         ABBC
        reitve, torej take, ki so mone pri enem samem t, ne pa pri      A00C
        celem intervalu [t, t + 1] ali em e daljem.  Primer je          00 
        na sliki.) }
      for j := 1 to StStikovD do with StikiD[j] do DodajStik(ia, ib);
      StStikovD := 0;
      { Premaknimo se na novi, veji t.}
      t := tNovi;
    end; {if}
    SpremembeNaStiku(i, Stiki[i].ia, Stiki[i].ib);
  end; {while}
end; {NajmanjsaRazdalja2}

{-----------------------------------------------------------------------------}
{ Podprogram za risanje razporedov pravokotnikov.                             }
{-----------------------------------------------------------------------------}

procedure Narisi(var a: VrstaT; aL: integer;
                 var b: VrstaT; bL: integer;
                 ZamikX, RazmikY: integer);
const SirinaZaslona = 80;
var SirinaA, SirinaB, ZamikA, ZamikB, i, x0, x, y: integer;
    S: string; C: char;
begin
  if ZamikX > 0 then ZamikA := 0 else ZamikA := -ZamikX;
  if ZamikX < 0 then ZamikB := 0 else ZamikB := ZamikX;
  SirinaA := ZamikA; for i := 1 to aL do Inc(SirinaA, a[i].Sirina);
  SirinaB := ZamikB; for i := 1 to bL do Inc(SirinaB, b[i].Sirina);
  S := ''; while (Length(S) < SirinaA) or (Length(S) < SirinaB) do S := S + ' ';
  for y := 1 to RazmikY do begin
    { Zanimo s prazno vrstico. }
    for x := 1 to Length(S) do S[x] := ' ';
    { Notranjost pravokotnikov zapolnimo s pikicami. }
    x0 := ZamikA;
    for i := 1 to aL do begin
      if y <= a[i].Globina then 
        for x := 1 to a[i].Sirina do S[x0 + x] := '.';
      x0 := x0 + a[i].Sirina;
    end; {for i}
    x0 := ZamikB; 
    for i := 1 to bL do begin
      if y > RazmikY - b[i].Visina then 
        for x := 1 to b[i].Sirina do S[x0 + x] := ',';
      x0 := x0 + b[i].Sirina;
    end; {for i}
    { Robove pravokotnikov pa predstavimo s rkami in tevkami. }
    x0 := ZamikA;
    for i := 1 to aL do begin 
      C := Chr(Ord('A') + (i - 1) mod (Ord('Z') - Ord('A') + 1));
      if y <= a[i].Globina then for x := 1 to a[i].Sirina do 
        if (x = 1) or (x = a[i].Sirina) or (y = 1) or (y = a[i].Globina) then S[x0 + x] := C;
      x0 := x0 + a[i].Sirina;
    end; {for i}
    x0 := ZamikB;
    for i := 1 to bL do begin 
      C := Chr(Ord('0') + (i - 1) mod (Ord('9') - Ord('0') + 1));
      if y > RazmikY - b[i].Visina then for x := 1 to b[i].Sirina do 
        if (x = 1) or (x = b[i].Sirina) or (y = RazmikY) or (y = RazmikY - b[i].Visina + 1) then S[x0 + x] := C;
      x0 := x0 + b[i].Sirina;
    end; {for i}
    { Izpiimo to vrstico. }
    if Length(S) >= SirinaZaslona then Write(Copy(S, 1, SirinaZaslona))
    else WriteLn(S);
  end; {for y}
end; {Narisi}

{ Glavni del programa: podatke o obeh vrsticah preberimo 
  kar s standardnega vhoda.  Najprej naj bo navedeno tevilo
  pravokotnikov v zgornji vrstici, nato pari (irina, globina);
  nato pa tevilo pravokotnikov v spodnji vrstici in e pari
  (irina, viina) za te pravokotnike.  Viin za prvo vrstico
  in globin za drugo nima smisla podajati, saj na na problem
  tako ali tako ne vplivajo. }
var a, b: VrstaT; aL, bL, i, ZamikX, RazmikY: integer;
begin {StiskanjeVrstic}

  { Preberimo vhodne podatke. }
  Read(aL);
  for i := 1 to aL do with a[i] do 
    begin Visina := 0; Read(Sirina, Globina); end;
  Read(bL);
  for i := 1 to bL do with b[i] do 
    begin Globina:= 0; Read(Sirina, Visina); end;
  
  { Poiimo najmanji primerni razmik in nariimo to reitev. }
  RazmikY := NajmanjsaRazdalja(a, aL, b, bL);
  WriteLn('Potrebni razmik med osnovnicama brez zamikanja: ', RazmikY);
  Narisi(a, aL, b, bL, 0, RazmikY);
  
  { Poiimo e najmanji razmik pri dovoljenem zamikanju. }
  WriteLn('-----------------------------------------------------------------');
  NajmanjsaRazdalja2(a, aL, b, bL, ZamikX, RazmikY);
  WriteLn('-----------------------------------------------------------------');
  WriteLn('Najbolja reitev z zamikanjem: ZamikX = ', ZamikX, ', RazmikY = ', RazmikY);
  Narisi(a, aL, b, bL, ZamikX, RazmikY);

end. {StiskanjeVrstic}
