program RegularniIzrazi; { RTK 1991.3.3 }
const MaxDolz = 100;
type Niz = packed array[1..MaxDolz] of char; 

{ Preprosta reitev z rekurzijo.  asovna zahtevnost je
  lahko eksponentna v odvisnosti od doline niza N. }
function Ujemanje(N, V: Niz; Nd, Vd: integer): boolean; 

  function Poskus(Ni, Vi: integer): boolean; 
  var Uspesno: boolean; 
  begin
    if Vi > Vd then 
      { Prili smo do konca vzorca -- ali tudi do konca niza? }
      Poskus := Ni > Nd 
    else if V[Vi] = '*' then begin 
      { Poskusimo z zvedico pokriti 0, 1, 2, 3, ... znakov niza. }
      Ni := Ni - 1; 
      repeat 
        Ni := Ni + 1; Uspesno := Poskus(Ni, Vi + 1); 
      until Uspesno or (Ni > Nd); 
      Poskus := Uspesno; 
    end else if (Ni <= Nd) and ((V[Vi] = '?') or (V[Vi] = N[Ni])) then 
      { Trenutni znak vzorca se ujema s trenutnim znakom niza. }
      Poskus := Poskus(Ni + 1, Vi + 1)
    else 
      { e pridemo do sem, pomeni, da trenutni znak vzorca ni niti
        vpraaj niti zvezdica, poleg tega pa se trenutni znak niza
        ne ujema z njim.  Torej se V[Vi..Vd] in N[Ni..Nd] ne ujemata. }
      Poskus := false;
  end; {Poskus} 

begin
  Ujemanje := Poskus(1, 1); 
end; {Ujemanje}

{ Izboljana razliica prejnje reitve: memoizacija.
  Podprogram Poskus vrednosti, ki jih vraa, shranjuje 
  v tabelo in jih kasneje prebere od tam, e jih spet
  potrebuje, tako da jih ni treba raunati ponovno. }
function Ujemanje2(N, V: Niz; Nd, Vd: integer): boolean; 
type ElementT = (Neznano, Neujemanje, Ujemanje);
var Tabela: array[1..MaxDolz + 1, 1..MaxDolz + 1] of ElementT;

  function Poskus(Ni, Vi: integer): boolean; 
  var Uspesno, Konec: boolean; Ni0, Vi0: integer;
  begin
    Assert((1 <= Ni) and (Ni <= Nd + 1));
    Assert((1 <= Vi) and (Vi <= Vd + 1));
    { Mogoe smo ujemanje za ta par (Ni, Vi) e kdaj preverili. }
    if Tabela[Ni, Vi] <> Neznano then 
      begin Poskus := Tabela[Ni, Vi] = Ujemanje; exit; end;
    { Preverimo ujemanje na obiajen nain, kot pri prejnji reitvi. }
    Konec := false; Ni0 := Ni; Vi0 := Vi;
    repeat 
      if Vi > Vd then begin 
        Konec := true; Uspesno := Ni > Nd 
      end else if V[Vi] = '*' then begin 
        Ni := Ni - 1; 
        repeat 
          Ni := Ni + 1; Uspesno := Poskus(Ni, Vi + 1); 
        until Uspesno or (Ni > Nd); 
        Konec := true;
      end else if (Ni <= Nd) and ((V[Vi] = '?') or (V[Vi] = N[Ni])) then begin 
        Ni := Ni + 1; Vi := Vi + 1 
      end else begin 
        Konec := true; Uspesno := false
      end; {if}
    until Konec; 
    { Shranimo vrednost funkcije v tabelo. }
    if Uspesno then Tabela[Ni0, Vi0] := Ujemanje
    else Tabela[Ni0, Vi0] := Neujemanje;
    Poskus := Uspesno;
  end; {Poskus} 

var Ni, Vi: integer;
begin
  for Ni := 1 to Nd + 1 do for Vi := 1 to Vd + 1 do 
    Tabela[Ni, Vi] := Neznano;
  Ujemanje2 := Poskus(1, 1); 
end; {Ujemanje2}

{ Izboljana razliica prejnje reitve: dinamino programiranje.
  To, kar so bili doslej rekurzivni klici podprograma Poskus,
  bomo tu izvajali sistematino po padajoih Vi in pri vsakem Vi
  po padajoih Ni.  Zato tudi ni treba hraniti cele tabele, pa
  pa je dovolj ena vrstica. }
function Ujemanje3(N, V: Niz; Nd, Vd: integer): boolean; 
var Tabela: array[1..MaxDolz + 1] of boolean;
    Ni, Vi: integer; B, C: boolean; 
begin
  { Ujemanje med V[Vd + 1..Vd] (= prazen niz) in N[Ni..Nd]. }
  Tabela[Nd + 1] := True;
  for Ni := Nd downto 1 do Tabela[Ni] := False;
  { Ujemanje med daljimi kosi vzorca in N-jevimi podnizi. }
  for Vi := Vd downto 1 do begin
    B := Tabela[Nd + 1]; Tabela[Nd + 1] := B and (V[Vi] = '*');
    for Ni := Nd downto 1 do begin
      { V B je trenutno ujemanje med V[Vi + 1..Vd] in N[Ni + 1..Nd]. 
        V Tabela[Ni] je ujemanje med V[Vi + 1..Vd] in N[Ni..Nd]. 
        V Tabela[Ni + 1] je ujemanje med V[Vi..Vd] in N[Ni + 1..Nd]. 
        Izraunajmo ujemanje med V[Vi..Vd] in N[Ni..Nd] in ga shranimo v C. }
      if V[Vi] = '*' then C := Tabela[Ni + 1] or Tabela[Ni] 
      else if V[Vi] = '?' then C := B
     else C := B and (V[Vi] = N[Ni]);
      { Ujemanje med V[Vi + 1..Vd] in N[Ni..Nd] si zapomnimo v B,
        pravkar izraunano vrednost C pa vpiimo v tabelo. }
      B := Tabela[Ni]; Tabela[Ni] := C;
      {WriteLn('Ujemanje(', Copy(N, Ni, Nd - Ni + 1), ', ', Copy(V, Vi, Vd - Vi + 1), ') = ', C);}
    end; {for Ni}
  end; {for Vi}
  Ujemanje3 := Tabela[1];
end; {Ujemanje3}

var Alg: integer; B: boolean; N, V: string;
begin {RegularniIzrazi}
  if ParamCount < 3 then begin
    WriteLn('Uporaba: rtk91s3n3 niz vzorec n');
    WriteLn('n je tevilka algoritma: 1 = rekurzija, ');
    WriteLn('2 = memoizacija, 3 = dinamino programiranje');
    exit;
  end; {if}
  N := ParamStr(1); V := ParamStr(2);
  if ParamStr(3) = '1' then Alg := 1
  else if ParamStr(3) = '2' then Alg := 2
  else Alg := 3;
  case Alg of 
  1: B := Ujemanje(N, V, Length(N), Length(V));
  2: B := Ujemanje2(N, V, Length(N), Length(V));
  else B := Ujemanje3(N, V, Length(N), Length(V));
  end; {case}
  Write('Ujemanje', Alg, '("', N, '", "', V, '") = ');
  if B then WriteLn('DA') else WriteLn('NE');
end. {RegularniIzrazi}
