program OsvezevanjeOken; { RTK 1992.3.4 }
type
  PravokotnikP = ^PravokotnikT;
  PravokotnikT = record
    x1, y1, x2, y2: integer; { koordinate }
    Nasl: PravokotnikP; { za povezovanje v sezname }
  end; {PravokotnikT}

function Max(A, B: integer): integer;
  begin if A < B then Max := B else Max := A; end;
function Min(A, B: integer): integer;
  begin if A > B then Min := B else Min := A; end;

function NovPravokotnik(x1, y1, x2, y2: integer; Nasl: PravokotnikP): PravokotnikP;
var P: PravokotnikP;
begin
  New(P);
  {WriteLn('- Nov(', x1, ', ', y1, ', ', x2, ', ', y2, ')');}
  P^.x1 := x1; P^.x2 := x2; P^.y1 := y1; P^.y2 := y2;
  P^.Nasl := Nasl; NovPravokotnik := P;
end; {NovPravokotnik}

{ V seznam S doda pravokotnike, ki jih dobimo, ko odreemo B od A. }
procedure OdreziDvaPravokotnika(A, B: PravokotnikT; var S: PravokotnikP);
begin
  {WriteLn('Odrezi (', B.x1, ', ', B.y1, ', ', B.x2, ', ', B.y2, ') od (', A.x1, ', ', A.y1, ', ', A.x2, ', ', A.y2, '):');}
  { Mogoe B sploh ni ne odree od A-ja.  Potem ostane kar cel A. }
  if (B.x2 <= A.x1) or (A.x2 <= B.x1) 
  or (B.y2 <= A.y1) or (A.y2 <= B.y1) then 
    begin S := NovPravokotnik(A.x1, A.y1, A.x2, A.y2, S); exit; end;
  { Drugae pa je razlika A - B unija nekaj manjih pravokotnikov. }
  if B.y1 > A.y1 then 
    { Tisti del A-ja, ki je v celoti nad B-jem. }
    S := NovPravokotnik(A.x1, A.y1, A.x2, B.y1, S);
  if B.y2 < A.y2 then
    { Tisti del A-ja, ki je v celoti pod B-jem. }
    S := NovPravokotnik(A.x1, B.y2, A.x2, A.y2, S);
  if B.x1 > A.x1 then
    { Tisti del A-ja, ki je levo ob B-ja (in na istih y-koordinatah).
      Presek intervalov (A.y1..A.y2-1) in (B.y1..B.y2-1) gotovo ni prazen,
      saj bi v tem primeru e prvi stavek if v tem podprogramu uspel
      in sploh ne bi prili do sem. }
    S := NovPravokotnik(A.x1, Max(A.y1, B.y1), B.x1, Min(A.y2, B.y2), S);
  if B.x2 < A.x2 then
    { Tisti del A-ja, ki je desno ob B-ju (in na istih y-koordinatah). }
    S := NovPravokotnik(B.x2, Max(A.y1, B.y1), A.x2, Min(A.y2, B.y2), S);
end; {OdreziDvaPravokotnika}

{ Odree pravokotnik B od vseh pravokotnikov iz seznama S.
  Vrne kazalec na seznam dobljenih manjih pravokotnikov,
  seznam S pa pri tem pobrie. }
function OdreziOdVecPravokotnikov(S: PravokotnikP; B: PravokotnikT): PravokotnikP;
var Rezultat, Nasl: PravokotnikP; 
begin
  Rezultat := nil;
  while S <> nil do begin
    OdreziDvaPravokotnika(S^, B, Rezultat);
    Nasl := S^.Nasl; Dispose(S); S := Nasl;
  end; {while}
  OdreziOdVecPravokotnikov := Rezultat;
end; {OdreziOdVecPravokotnikov}

const MaxOken = 20;
var 
  { Okna[1] je tisto na vrhu, Okna[StOken] pa tisto na dnu. }
  Okna: array[1..MaxOken] of PravokotnikT;
  StOken: integer;

function OsveziOkno(StOkna: integer): PravokotnikP;
var i: integer; Rezultat: PravokotnikP;
begin
  with Okna[StOkna] do 
    Rezultat := NovPravokotnik(x1, y1, x2, y2, nil);
  for i := 1 to StOkna - 1 do
    Rezultat := OdreziOdVecPravokotnikov(Rezultat, Okna[i]);
  OsveziOkno := Rezultat;
end; {OsveziOkno}

const MaxX = 70; MaxY = 40;
type ZaslonT = array[0..MaxY - 1, 0..MaxX - 1] of integer;

{ Pobarva piksle, ki pripadajo danemu pravokotniku, z vrednostjo St. 
  e je PricakujSt >= 0 in ima kaken od teh pikslov pred tem
  kakno drugo vrednost kot PricakujSt, javi napako.  S tem lahko
  preverimo, e OsveziOkno res vrne seznam *disjunktnih* pravokotnikov. }
procedure NarisiPravokotnik(var Z: ZaslonT; P: PravokotnikT; St, PricakujSt: integer);
var x, y: integer;
begin
  Assert((0 <= P.x1) and (P.x1 < P.x2) and (P.x2 <= MaxX));
  Assert((0 <= P.y1) and (P.y1 < P.y2) and (P.y2 <= MaxY));
  for y := P.y1 to P.y2 - 1 do for x := P.x1 to P.x2 - 1 do begin
    if (PricakujSt >= 0) and (Z[y, x] <> PricakujSt) then
      begin WriteLn('Napaka!'); Halt; end;
    Z[y, x] := St;
  end; {for y, x}
end; {NarisiPravokotnik}

{ Narie na zaslon vse pravokotnike iz seznama P.
  Ta seznam pri tem pobrie! }
procedure NarisiPravokotnike(var Z: ZaslonT; P: PravokotnikP; St, PricakujSt: integer);
var Nasl: PravokotnikP;
begin
  while P <> nil do begin
    NarisiPravokotnik(Z, P^, St, PricakujSt);
    Nasl := P^.Nasl; Dispose(P); P := Nasl;
  end; {while}
end; {NarisiPravokotnike}

procedure Preizkus(Narisi: boolean);
var Z1, Z2: ZaslonT; i, x, y, cx, cy: integer;
begin
  Assert((0 <= StOken) and (StOken <= MaxOken));
  { Pripravimo prazni tabeli. }
  for y := 0 to MaxY - 1 do for x := 0 to MaxX - 1 do 
    begin Z1[y, x] := 0; Z2[y, x] := 0; end;
  { Na Z1 riimo okna preprosto od spodnjih k zgornjim. }
  for i := StOken downto 1 do 
    NarisiPravokotnik(Z1, Okna[i], i, -1);
  { Na Z2 pa riimo okna od zgornjih proti spodnjim.  Pri tem
    od vsakega odreimo tista, ki so nad njim, da jih ne bomo povozili. }
  for i := 1 to StOken do 
    NarisiPravokotnike(Z2, OsveziOkno(i), i, 0);
  { Oba zaslona bi se morala povsem ujemati. }
  for y := 0 to MaxY - 1 do for x := 0 to MaxX - 1 do 
    if Z1[y, x] <> Z2[y, x] then begin WriteLn('Napaka!'); Halt; end;
  { Vse je v redu, nariimo na zaslon. }
  if Narisi then begin
    cx := 0; cy := 0;
    for i := 1 to StOken do with Okna[i] do
      begin cx := Max(cx, x2); cy := Max(cy, y2); end;
    for y := 0 to cy - 1 do begin
      for x := 0 to cx - 1 do 
        if Z1[y, x] = 0 then Write('.')
        else Write(Chr(Ord('a') + Z1[y, x] - 1));
      WriteLn;
    end; {for y}
  end; {if}
end; {Preizkus}

{ Preizkusi ve primerov z dvema oknoma (in razlinimi
  poloaji vrhnjega glede na spodnjega). }
procedure Preizkus2;
var x1, y1, x2, y2: integer;
begin
  StOken := 2;
  with Okna[2] do begin x1 := 5; y1 := 25; x2 := 10; y2 := 30; end;
  for x1 := 1 to 11 do for x2 := x1 + 1 to 12 do 
  for y1 := 1 to 11 do for y2 := y1 + 1 to 12 do begin
    Okna[1].x1 := x1; Okna[1].x2 := x2;
    Okna[1].y1 := y1 + 20; Okna[1].y2 := y2 + 20;
    Preizkus(false);
  end; {for}
end; {Preizkus2}

{ Nakljuno razmee nekaj oken. }
procedure Preizkus3(Narisi: boolean);
var i: integer;
begin
  StOken := MaxOken;
  for i := 1 to StOken do begin
    Okna[i].x1 := Random(MaxX);
    Okna[i].x2 := Okna[i].x1 + 1 + Random(MaxX - Okna[i].x1);
    Okna[i].y1 := Random(MaxY);
    Okna[i].y2 := Okna[i].y1 + 1 + Random(MaxY - Okna[i].y1);
  end; {for}
  Preizkus(Narisi);
end; {Preizkus3}

var i: integer;
begin {OsvezevanjeOken}
  { Sistematini testi z dvema oknoma. }
  Preizkus2;
  { Nakljuni testi z ve okni. }
  for i := 1 to 10 do Preizkus3(i = 1);
end. {OsvezevanjeOken}
