program SimulatorPredpomnilnika; { RTK 1996.2.2 }

  { Vmesnik, ki ga vidijo podprogrami za upravljanje predpomnilnika. }
  type Podatki = string;
  procedure ZapisiBlokNaDisk(StBloka: integer; P: Podatki); forward;
  procedure PreberiBlokZDiska(StBloka: integer; var P: Podatki); forward;

{-----------------------------------------------------------------------------}
{ Upravitelj predpomnilnika                                                   }
{ (to je tisto, o emer so morali pri tej nalogi razmiljati tekmovalci)      }
{-----------------------------------------------------------------------------}

const MaksPPBlokov = 20;
type
  BlokT = record
    StBloka: integer;
    P: Podatki;
    ZadnjiDostop: integer;
    Spremenjen: boolean;
  end; {BlokT}
var
  { Namesto razprene tabele ali esa podobnega imamo kar tole navadno 
    tabelo, ker je predpomnilnik pri nai simulaciji majhen. }
  Predpomnilnik: array[1..MaksPPBlokov] of BlokT;
  { tevec dostopov do predpomnilnika.  Uporabljamo ga za
    prirejanje vrednosti ZadnjiDostop. }
  Cas: integer; 

{ e ga ni v predpomnilniku, vrne MaksPPBlokov + 1. }
function NajdiBlok(StBloka: integer): integer;
var i: integer;
begin
  i := 1;
  while i <= MaksPPBlokov do  
    if Predpomnilnik[i].StBloka = StBloka then break
    else i := i + 1;
  NajdiBlok := i;
end; {NajdiBlok}

function ZagotoviProstorZaEnBlok: integer;
var i, bi: integer;
begin
  { Poiimo kak prost blok v pomnilniku, e pa so vsi zasedeni, 
    poiimo takega, ki e najdlje ni bil uporabljen. }
  bi := 1; 
  for i := 1 to MaksPPBlokov do 
    if Predpomnilnik[i].StBloka = -1 then 
      begin bi := i; break; end
    else if Predpomnilnik[i].ZadnjiDostop < Predpomnilnik[bi].ZadnjiDostop then
      bi := i;
  { Vrzimo ta blok iz predpomnilnika; e je bil spremenjen, 
    ga moramo shraniti. }
  if (Predpomnilnik[bi].StBloka >= 0) and Predpomnilnik[bi].Spremenjen then
    ZapisiBlokNaDisk(Predpomnilnik[bi].StBloka, Predpomnilnik[bi].P);
  ZagotoviProstorZaEnBlok := bi;
end; {ZagotoviProstorZaEnBlok}

procedure ZapisiBlok(StBloka: integer; P: Podatki);
var i: integer;
begin
  i := NajdiBlok(StBloka); Cas := Cas + 1; 
  { e ga e ni v predpomnilniku, pripravimo zanj nov zapis. }
  if i > MaksPPBlokov then begin
    i := ZagotoviProstorZaEnBlok;
    Predpomnilnik[i].StBloka := StBloka;
  end; {if}
  { Osveimo podatke, as dostopa in prigimo zastavico Spremenjen. }
  Predpomnilnik[i].P := P;
  Predpomnilnik[i].ZadnjiDostop := Cas;
  Predpomnilnik[i].Spremenjen := true;
end; {ZapisiBlok}

procedure PreberiBlok(StBloka: integer; var P: Podatki);
var i: integer;
begin
  i := NajdiBlok(StBloka); Cas := Cas + 1; 
  { e ga e ni v predpomnilniku, ga preberimo z diska. }
  if i > MaksPPBlokov then begin
    i := ZagotoviProstorZaEnBlok;
    Predpomnilnik[i].StBloka := StBloka;
    PreberiBlokZDiska(StBloka, Predpomnilnik[i].P);
    Predpomnilnik[i].Spremenjen := false;
  end; {if}
  { Osveimo as dostopa, vrnimo podatke. }
  P := Predpomnilnik[i].P;
  Predpomnilnik[i].ZadnjiDostop := Cas; 
end; {SpremeniBlok}

procedure ZapisiVseNaDisk;
var i: integer;
begin
  for i := 1 to MaksPPBlokov do with Predpomnilnik[i] do 
    if (StBloka >= 0) and Spremenjen then 
      begin ZapisiBlokNaDisk(StBloka, P); Spremenjen := false; end;
end; {ZapisiVseNaDisk}

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira dostop do fizinega diska in si izmilja       }
{ zahtevke, ki prihajajo do upravitelja predpomnilnika.                       }
{-----------------------------------------------------------------------------}

const 
  MaksZadnjih = 10;
  MaksBlokov = 100;
type
  ZadnjiT = record
    Zadnji: array[1..MaksZadnjih] of integer;
    StZadnjih: integer;
  end; {record}

{ Izmisli si, kateri blok bo program potreboval kot
  naslednjega.  Bloki, ki jih je uporabljal v zadnjem
  asu, imajo veje monosti, da bodo spet na vrsti. 
  Veje ko je BremeZgodovine, bolj se ponavljajo stari 
  zahtevki (do istih blokov kot pred nedavnim) -- in bolj 
  se nam splaa imeti predpomnilnik. }
function NovZahtevek(var Z: ZadnjiT): integer;
const BremeZgodovine = 4;
var i, Blok: integer;
begin
  i := Random((BremeZgodovine + 1) * MaksZadnjih) + 1;
  if i <= BremeZgodovine * Z.StZadnjih then 
    Blok := Z.Zadnji[(i + BremeZgodovine - 1) div BremeZgodovine]
  else Blok := Random(MaksBlokov) + 1;
  if Z.StZadnjih < MaksZadnjih then Inc(Z.StZadnjih)
  else for i := 1 to Z.StZadnjih - 1 do 
    Z.Zadnji[i] := Z.Zadnji[i + 1];
  Z.Zadnji[Z.StZadnjih] := Blok;
  NovZahtevek := Blok;
end; {NovZahtevek}

var 
  Disk: array[1..MaksBlokov] of Podatki;
  { Kaken bi bil disk, e ne bi bilo predpomnilnika. }
  DiskBrezPP: array[1..MaksBlokov] of Podatki; 
  { Malo statistike. }
  StBranj, StPisanj, StBranjPP, StPisanjPP: integer;

procedure ZapisiBlokNaDisk(StBloka: integer; P: Podatki); 
begin
  Disk[StBloka] := P;
  WriteLn('ZapisiBlokNaDisk(', StBloka, ', "', P, '")');
  StPisanj := StPisanj + 1;
end; {ZapisiBlokNaDisk}

procedure PreberiBlokZDiska(StBloka: integer; var P: Podatki); 
begin
  P := Disk[StBloka];
  WriteLn('PreberiBlokZDiska(', StBloka, ', "', P, '")');
  StBranj := StBranj + 1;
end; {PreberiBlokZDiska}

function ToStr(i: integer): string;
var S: string; 
begin
  Str(i, s); ToStr := s; 
end; {ToStr}

var i, j: integer; Z: ZadnjiT; P: Podatki;
begin {SimulatorPredpomnilnika}

  { Inicializacija predpomnilnika. }
  for i := 1 to MaksPPBlokov do 
    Predpomnilnik[i].StBloka := -1;
  Cas := 0;
  
  { Formatirajmo disk :). }
  for i := 1 to MaksBlokov do begin
    Disk[i] := ToStr(i) + '-nedotaknjen'; 
    DiskBrezPP[i] := Disk[i];
  end; {for i}

  { Izmislimo si zaporedje zahtevkov in izpisujmo, kaj se dogaja. }  
  Z.StZadnjih := 0;
  for j := 1 to 1000 do begin
  
    i := NovZahtevek(Z);
    if Random(10) < 4 then begin { Zahtevajmo pisanje. }
      StPisanjPP := StPisanjPP + 1;
      P := ToStr(i) + '-spremenjen-' + ToStr(StPisanjPP);
      WriteLn('ZapisiBlok(', i, ', "', P, '")');
      ZapisiBlok(i, P);
      DiskBrezPP[i] := P;
    end 
    
    else begin { Zahtevajmo branje. }
      StBranjPP := StBranjPP + 1;
      WriteLn('PreberiBlok(', i, ')');
      PreberiBlok(i, P);
      WriteLn('PreberiBlok(', i, ') je vrnil "', P, '"');
      { Ali smo dobili to, kar smo nazadnje zapisali v ta blok? }
      Assert(DiskBrezPP[i] = P);
    end; 
    
    WriteLn('tevilo branj: ', StBranjPP, ' (fizino: ', StBranj, 
            '), pisanj: ', StPisanjPP, ' (fizino: ', StPisanj, ').');
    Write('Predpomnilnik:');
    for i := 1 to MaksPPBlokov do with Predpomnilnik[i] do if StBloka >= 0 then begin
      Write(' ', StBloka);
      if Spremenjen then Write('*');
      Write('/', ZadnjiDostop);
    end; {if, with, for i}
    WriteLn;
  
  end; {for j}
  
  WriteLn('Praznim predpomnilnik...');
  ZapisiVseNaDisk;
  { Preverimo, e je vse v redu -- e so na disku brez
    predpomnilnika enaki podatki kot na tistem s predpomnilnikom. }
  for i := 1 to MaksBlokov do 
    Assert(DiskBrezPP[i] = Disk[i]);

end. {SimulatorPredpomnilnika}
