program SimulatorPosiljanjaZetona; { RTK 1995.3.4 }

  type SporociloT = integer;
  { Vmesnik, ki ga vidi podprogram KdoSem. 
    tevilka MojaSt se uporablja le za izpis na zaslon, 
    da uporabnik laje sledi simulaciji.  V resnici seveda,
    e bi vsak raunalnik e poznal neko svojo enolino
    tevilko, bi bil cel postopek poiljanja etona odve,
    saj je njegov namen ravno ta, da dobi vsak raunalnik
    tako tevilko. }
  function Poslji(MojaSt, VSmer: integer; Sporocilo: SporociloT): boolean; forward;
  function Sprejmi(MojaSt: integer; var IzSmeri: integer; var Sporocilo: SporociloT): boolean; forward;
  procedure NastaviSvojNaslov(MojaSt, Naslov: integer); forward; 
  const MaxRac = 100;
  const MaxSmeri = 11; 

type 
  StanjeT = (Zacetek, CakamPrvo, PosiljamNaprej, CakamVrnitev, Konec);
  { Ker moramo simulirati izvajanje podprograma KdoSem na ve
    raunalnikih naenkrat, ga bomo v resnici izvajali po kokih,
    v strukturi tipa KontekstT pa bo lahko hranil svoje
    lokalne spremenljivke in podobne zadeve. }
  KontekstT = record
    PrvotnaSmer, StevecSmeri, Naslov: integer; 
    PoznaneSmeri: array[1..MaxSmeri] of boolean; 
    { Prek polja Stanje si podprogram KdoSem zapomni, 
      kaj mora poeti ob naslednjem klicu. }
    Stanje: StanjeT;
  end; {KontekstT}
const Ime: array[StanjeT] of string = ('Zac', 'Prv', 'Pos', 'Cak', 'Kon');  
 
procedure KdoSem(StRac: integer; var Kontekst: KontekstT);
var Smer: integer;
begin
  with Kontekst do begin
  
  if Stanje = Zacetek then begin
    { Tole bi bila lahko zanka for, vendar StevecSmeri ni
      lokalna spremenljivka in so s tem vasih teave. }
    StevecSmeri := 1; while StevecSmeri <= MaxSmeri do
      begin PoznaneSmeri[StevecSmeri] := false; Inc(StevecSmeri); end;
    Stanje := CakamPrvo; 
  end; {Zacetek}
  
  if Stanje = CakamPrvo then begin
    if not Sprejmi(StRac, PrvotnaSmer, Naslov) then exit;
    if PrvotnaSmer = 0 then Naslov := 1 
    else begin 
      Naslov := Naslov + 1; 
      PoznaneSmeri[PrvotnaSmer] := true; 
    end; {if}
    NastaviSvojNaslov(StRac, Naslov); 
    Stanje := PosiljamNaprej;
  end; {if}
  
  if Stanje = PosiljamNaprej then begin
    StevecSmeri := 1; while StevecSmeri <= MaxSmeri do begin
      if not PoznaneSmeri[StevecSmeri] then begin
        PoznaneSmeri[StevecSmeri] := true; 
        if Poslji(StRac, StevecSmeri, Naslov) then 
          begin Stanje := CakamVrnitev; exit; end;
      end; {if}
      Inc(StevecSmeri);
    end; {while}
    { e pridemo do sem, pomeni, da smo li prek vseh smeri
      in pri vsaki videli, da je e znana -- torej smo konali. }
    Poslji(StRac, PrvotnaSmer, Naslov); 
    Stanje := Konec;
  end; {if}
  
  if Stanje = CakamVrnitev then begin
    { Pred asom smo poslali eotn v smer StevecSmeri
      in zdaj akamo, da bo priel nazaj od tam.  Medtem
      lahko pride e iz kakne druge smeri in v tem primeru
      ga le odbijemo nazaj. }
    if not Sprejmi(StRac, Smer, Naslov) then exit;
    if Smer <> StevecSmeri then begin { odbijmo eton nazaj }
      PoznaneSmeri[Smer] := true; 
      Poslji(StRac, Smer, Naslov); 
    end else
      { eton se je vrnil, lahko ga bomo poslali e kam. }
      Stanje := PosiljamNaprej;
  end; {if}
  
  end; {with Kontekst}
end; {KdoSem} 

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira dogajanje v mrei.                             }
{ To je zelo podobno kot pri rtk95s2n4.pas.                                   }
{-----------------------------------------------------------------------------}

type
  VrstaP = ^VrstaT;
  VrstaT = record
    S: SporociloT;
    IzSmeri: integer;
    Nasl: VrstaP;
    KdajPride: integer;
  end; {VrstaT}
var
  { V resnici je miljeno, da so veljavne smeri od 1 naprej,
    smer 0 pa vedno vsebuje vrednost 0.  Tudi tevilke
    raunalnikov naj bi bile le od 1 naprej, tevilka 0
    pa predstavlja uporabnika. }
  Povezave: array[0..MaxRac, 0..MaxSmeri] of 0..MaxRac;
  Glave, Repi: array[0..MaxRac] of VrstaP;
  Kontekst: array[1..MaxRac] of KontekstT;
  Naslovi: array[1..MaxRac] of integer;
  Cas: integer;

function Poslji(MojaSt, VSmer: integer; 
                Sporocilo: SporociloT): boolean; 
var Komu: integer; P: VrstaP;
begin
  { Ali ta smer sploh obstaja?  VSmer = 0 je poseben primer,
    ki pomeni poiljanje uporabniku ("raunalnik" t. 0). }
  Komu := Povezave[MojaSt, VSmer];
  if (Komu = 0) and (VSmer > 0) then begin Poslji := false; exit; end;
  New(P); P^.S := Sporocilo; 
  { Ker na simulator simulira raunalnike po vrsti, bi
    se lahko zgodilo, da npr. raunalnik i polje sporoilo
    raunalniku i+1, ta pa ga prevzame e v okviru istega
    asovnega koraka.  To ni le nerealistino, pa pa obiajno
    tudi povzroi, da si ustvari raunalnik i+1 napano 
    podobo o tem, v kateri smeri gre najkraja pot do nekaterih
    drugih raunalnikov.  Zato uporabljamo polje KdajPride,
    da podprogram Sprejmi ne bo prezgodaj prevzemal sporoil. }
  P^.KdajPride := Cas + 1;
  { Iz katere svoje smeri bo prejemnik dobil to sporoilo? }
  P^.IzSmeri := 0;
  while Povezave[Komu, P^.IzSmeri] <> MojaSt do Inc(P^.IzSmeri);
  { Dodajmo sporoilo na konec prejemnikove vrste. }
  if Repi[Komu] = nil then Glave[Komu] := P
  else Repi[Komu]^.Nasl := P;
  Repi[Komu] := P; P^.Nasl := nil; Poslji := true;
  WriteLn('[T = ', Cas:4, ']  ', MojaSt, ' poilja ', 
           Komu, ' sporoilo ', Sporocilo, '.');
end; {Poslji}

function Sprejmi(MojaSt: integer; var IzSmeri: integer; 
                 var Sporocilo: SporociloT): boolean;
var P: VrstaP; 
begin
  if Glave[MojaSt] = nil then begin Sprejmi := false; exit; end;
  if Glave[MojaSt]^.KdajPride > Cas then begin Sprejmi := false; exit; end;
  P := Glave[MojaSt]; Glave[MojaSt] := P^.Nasl;
  if P^.Nasl = nil then Repi[MojaSt] := nil;
  IzSmeri := P^.IzSmeri; Sporocilo := P^.S;
  Dispose(P); Sprejmi := true;
  WriteLn('[T = ', Cas:4, ']  ', MojaSt, ' prejme od ', 
          Povezave[MojaSt, IzSmeri], ' sporoilo ', Sporocilo, '.');
end; {Sprejmi}

procedure NastaviSvojNaslov(MojaSt, Naslov: integer); 
begin
  Writeln('[T = ', Cas:4, ']  ', MojaSt, ' si pripie naslov ', Naslov, '.');
  Naslovi[MojaSt] := Naslov;
end; {NastaviSvojNaslov}

var SeKajTece: boolean; SteviloRacunalnikov, i, j, k: integer; T: text;
begin {SimulatorPosiljanjaZetona}

  { Preberimo opis povezav.  Uporabimo kar isto datoteko
    kot naloga 1995.2.4. }
  if ParamCount >= 1 then Assign(T, ParamStr(1))
  else Assign(T, 'rtk95s2n4.in');
  Reset(T); ReadLn(T, SteviloRacunalnikov);
  for i := 0 to SteviloRacunalnikov do 
    for k := 0 to MaxSmeri do Povezave[i, k] := 0;
  while true do begin
    ReadLn(T, i, j); if (i <= 0) or (j <= 0) then break;
    k := 1; while Povezave[i, k] > 0 do Inc(k);
    Povezave[i, k] := j;
    k := 1; while Povezave[j, k] > 0 do Inc(k);
    Povezave[j, k] := i;
  end; {while}
  Close(T);
  WriteLn('Povezave:');
  for i := 1 to SteviloRacunalnikov do begin
    Write(i:2, ':');
    for k := 1 to MaxSmeri do Write(' ', Povezave[i, k]:2);
    WriteLn;
  end; {for i}
  
  { Pripravimo se na izvajanje podprograma KamNajPosljem. }
  for i := 1 to SteviloRacunalnikov do begin 
    Kontekst[i].Stanje := Zacetek;
    Naslovi[i] := 0;
    Glave[i] := nil; Repi[i] := nil;
  end; {for}
  
  Cas := 0;

  { Koga bi mi kot uporabnik prvega obvestili? }
  Glave[0] := nil; Repi[0] := nil;
  WriteLn('[T = ', Cas:4, ']  Uporabnik polje eton raunalniku ', i, '.');
  i := Random(SteviloRacunalnikov) + 1;
  Povezave[0, 1] := i; 
  Poslji(0, 1, 12345);
  
  repeat
    SeKajTece := false;
    for i := 1 to SteviloRacunalnikov do 
      if Kontekst[i].Stanje <> Konec then begin
        KdoSem(i, Kontekst[i]);
        SeKajTece := SeKajTece or (Kontekst[i].Stanje <> Konec);
      end; {if}
    if Sprejmi(0, i, j) then begin
      Assert(i = 1);
      WriteLn('[T = ', Cas:4, ']  Uporabnik je dobil eton nazaj (z vrednostjo ', j, ').');
    end; {if}
    SeKajTece := SeKajTece or (Glave[0] <> nil);
      
    { Izpiimo malo statistike. }
    Write('[T = ', Cas:4, ']  Stanja in znani naslovi:');
    for i := 1 to SteviloRacunalnikov do begin
      Write(' ', i, ':', Ime[Kontekst[i].Stanje], '/');
      if Naslovi[i] = 0 then Write('-')
      else Write(Naslovi[i]);
    end; {for}
    WriteLn;
    Inc(Cas); 
  until not SeKajTece;
  
end. {SimulatorPosiljanjaZetona}
