program Protokol1; { RTK 1996.3.1 }
{ Ta program simulira dogajanje v omreju.
  Ker mora simulirati dva raunalnika naenkrat,
  izvaja njuna programa vedno po kokih 
  (podprogram Korak).  V strukturi RacunalnikT
  si zapomni stanje posameznega raunalnika (trenutni
  ukaz, lokalne spremenljivke).
    Prej ali slej udarijo na dan napake v protokolu.
  Program bo z velikimi rkami izpisal, da bi nek
  raunalnik ignoriral sporoilo oz. ne bi poslal
  sporoila, ki ga je vtipkal uporabnik. }

uses Crt;
const MaxVrsta = 10;
type
  SporociloT = (PosiljalBi, KarDaj, RajeNe, Podatki);
  StanjeT = (Osnovno, RadBiPosiljal, Posiljanje, ZahtevanSprejem, Sprejemanje);
  RacunalnikT = record
    Stanje: StanjeT;
    SprejemVkljucen: boolean;
    Vrsta: array[1..MaxVrsta] of SporociloT;
    Dolzina: integer;
  end; {RacunalnikT}
const 
  ImeSporocila: array[SporociloT] of string = ('PosiljalBi', 'KarDaj', 'RajeNe', 'Podatki');
  ImeStanja: array[StanjeT] of string = ('Osnovno', 'RadBiPosiljal', 'Posiljanje', 'ZahtevanSprejem', 'Sprejemanje');

var Rac: array[1..2] of RacunalnikT;
    ZadnjeStanje: string;


procedure PrikaziStanje;
var StRac, i: integer; S, T: string;
begin
  S := '';
  for StRac := 1 to 2 do with Rac[StRac] do begin
    Str(StRac, T); S := S + '[' + T + '.' + ImeStanja[Stanje] + '/';
    if SprejemVkljucen then S := S + 'vkljuen' 
    else S := S + 'izkljuen';
    for i := 1 to Dolzina do S := S + ' ' + ImeSporocila[Vrsta[i]];
    S := S + '] ';
  end; {for}
  if ZadnjeStanje <> S then 
    begin WriteLn(S); ZadnjeStanje := S; exit; end;
end; {PrikaziStanje}

{ V spremenljivki KajNaredil vrne true, e je kaj naredil
  (sprejel poslal, spremenil stanje ipd.) oz. e bi lahko
  kaj naredil (npr. e bi se uporabnik odloil poslati
  sporoilo).  e pa je v takem stanju, da na nekaj aka
  in tega nekaj e ni, vrne KajNaredil = false. }
procedure Korak(StRac: integer; var KajNaredil: boolean);

  function Sprejmi: SporociloT;
  var i: integer;
  begin
    with Rac[StRac] do begin
      Assert(Dolzina > 0);
      Sprejmi := Vrsta[1];
      for i := 2 to Dolzina do Vrsta[i - 1] := Vrsta[i];
      Dec(Dolzina);
    end; {with}
  end; {Sprejmi}
  
  procedure Poslji(S: SporociloT);
  begin
    with Rac[3 - StRac] do begin
      if Dolzina >= MaxVrsta then 
        begin WriteLn('[', StRac, '.', ImeStanja[Rac[StRac].Stanje], ']  Poslal sporoilo ', ImeSporocila[S], ', vendar ima drugi raunalnik prepolno vrsto.'); exit; end;
      WriteLn('[', StRac, '.', ImeStanja[Rac[StRac].Stanje], ']  Poslal sporoilo ', ImeSporocila[S], '.');
      Inc(Dolzina); Vrsta[Dolzina] := S;
    end; {with}
  end; {Poslji}

var S: SporociloT;
begin
  KajNaredil := true;
  with Rac[StRac] do begin
    case Stanje of
    
    Osnovno: begin
      { Pri nai simulaciji dejstvo, da je sporoilo v vrsti,
        v resnici pomeni le, da je bilo e poslano, ne pa, 
        da je e prispelo.  Torej nismo dolni nemudoma prevzeti
        sporoila, im se pojavi v vrsti -- e ga ne prevzamemo
        takoj, s tem simuliramo monost, da se zakasni na poti. }
      if (Dolzina > 0) and (Random(10) < 5) then begin
        S := Sprejmi;
        if S <> PosiljalBi then begin WriteLn('[', StRac, '.', ImeStanja[Stanje], ']  IGNORIRAM sporoilo ', ImeSporocila[S], '.'); exit; end;
        WriteLn('[', StRac, '.', ImeStanja[Stanje], ']  Prejel sporoilo ', ImeSporocila[S], '.');
        Stanje := ZahtevanSprejem; exit;
      end; {if}
      case Random(10) of
      1: begin 
        SprejemVkljucen := not SprejemVkljucen; Write('[', StRac, '.', ImeStanja[Stanje], ']  Postavil SprejemVkljucen na ');
        if SprejemVkljucen then Write('true') else Write('false');
        WriteLn('.');
      end; 
      2: begin Poslji(PosiljalBi); Stanje := RadBiPosiljal; end;
      end; {case}
    end; {Osnovno}
    
    RadBiPosiljal: begin
      if Dolzina <= 0 then begin KajNaredil := false; exit; end;
      S := Sprejmi; Stanje := Osnovno; 
      case S of
      RajeNe: WriteLn('[', StRac, '.', ImeStanja[Stanje], ']  Prejel sporoilo ', ImeSporocila[S], '.'); 
      KarDaj: begin 
        WriteLn('[', StRac, '.', ImeStanja[Stanje], ']  Prejel sporoilo ', ImeSporocila[S], '.'); 
        Stanje := Posiljanje; end;
      else WriteLn('[', StRac, '.', ImeStanja[Stanje], ']  IGNORIRAM sporoilo ', ImeSporocila[S], ', uporabnikovega sporoila NE BOM POSLAL.'); 
      end; {case}
    end; {RadBiPosiljal}
    
    Posiljanje: begin
      Poslji(Podatki); Stanje := Osnovno; 
    end; {Posiljanje}
    
    ZahtevanSprejem: begin
      if SprejemVkljucen then begin 
        Poslji(RajeNe); Stanje := Osnovno;
      end else begin 
        Poslji(KarDaj); Stanje := Sprejemanje;
      end; {if}
    end; {ZahtevanSprejem}
    
    Sprejemanje: begin
      if Dolzina <= 0 then begin KajNaredil := false; exit; end;
      S := Sprejmi; 
      Write('[', StRac, '.', ImeStanja[Stanje], ']  Prejel sporoilo ', ImeSporocila[S]);
      if S <> Podatki then Write(' (MORALO BI BITI Podatki!)');
      WriteLn('.'); Stanje := Osnovno;
    end; {Sprejemanje}
      
    end; {case}
  end; {with}
end; {Korak}

var StRac: integer; KajNaredil: boolean; c: char;
begin {Protokol1}

  WriteLn('Simulator omreja');
  WriteLn('Pritisni P za pavzo, Q za izhod.');
  
  for StRac := 1 to 2 do with Rac[StRac] do begin { Inicializacija. }
    Stanje := Osnovno; SprejemVkljucen := true; Dolzina := 0; 
  end; {with, for}
  ZadnjeStanje := '';
  
  while true do begin
  
    PrikaziStanje;
    
    StRac := 1 + Random(2);
    Korak(StRac, KajNaredil);
    
    if not KajNaredil then begin
      Korak(3 - StRac, KajNaredil);
      if KajNaredil = false then 
        begin WriteLn('Nobeden od raunalnikov niesar ne naredi.  Sistem je zmrznil.'); break; end;
    end; {if}
    
    Delay(100);
    if KeyPressed then begin
      c := UpCase(ReadKey); if c = #0 then c := ReadKey;
      if (c = 'Q') or (c = 'X') then break;
      if c = 'P' then begin
        Write('(pavza)', #13);
        c := UpCase(ReadKey); if c = #0 then c := ReadKey;
        if (c = 'Q') or (c = 'X') then break;
      end; {if}
    end; {if}
    
  end; {with}
end. {Protokol1}


  