program Naslovi; { RTK 1998.2.4 }
{ Ta program simulira paralelno pretvarjanje mnoice
  imen v naslove.  Namesto nizov so imena in naslovi
  predstavljeni kar s tevilkami (od 10001 naprej).
  Ker se seveda ne bomo zares pogovarjali s streniki DNS
  ali im podobnim, naj bo naslov kar enak imenu. }

const 
  StProcesorjev = 8; 
  ShranjenNaslovM = 50; { dolina tabele ShranjenNaslov } 
type Niz = longint; 

  function Obdelaj(Ime: Niz): integer; forward; 
  function PoberiRezultat(var Naslov: Niz): integer; forward; 

{ To je podprogram, ki ga naloga zahteva od tekmovalcev. 
  Namesto da bi bral imena iz datoteke, bo uporabil kot
  imena kar tevila od 10001 do 10000 + Koliko. }
procedure ObdelajZaporedjeNaslovov(Koliko: integer);
var 
  ImeRac: Niz; 
  { tabela urejenih, e neizpisanih naslovov } 
  ShranjenNaslov: array[1..ShranjenNaslovM] of Niz; 
  { zaporedna tevilka, ki je v obdelavi na posameznem procesorju } 
  PripadajocaZapSt: array[1..StProcesorjev] of integer; 
  { indeks, kjer je shranjen naslov s trenutno najnijo zaporedno tevilko } 
  IndNajnizjeZapSt: integer; 
  { najveja zaporedna tevilka, ki jo je mono trenutno hraniti v tabeli ShranjenNaslov } 
  NajvecjaZapSt: integer;    
  ZapSt: integer;       { zaporedna tevilka prebranega podatka } 
  vObdelavi: integer;   { tevilo imen, ki so v paralelni obdelavi } 
  j: integer; 

  procedure IzpisiKarMores; 
  { Vzame en rezultat iz obdelave, ga uvrsti na pravo mesto v tabelo 
    ShranjenNaslov in izpie, kolikor se je nabralo urejenega dela 
    rezultatov v tej tabeli. }
  var j, Kam: integer; Naslov: Niz; 
  begin
    j := PoberiRezultat(Naslov); vObdelavi := vObdelavi - 1; 
    Kam := (PripadajocaZapSt[j] - 1) mod ShranjenNaslovM + 1; 
    ShranjenNaslov[Kam] := Naslov; 
    while ShranjenNaslov[IndNajnizjeZapSt] <> 0 do begin 
      WriteLn('Obdelal sem naslov ', ShranjenNaslov[IndNajnizjeZapSt], '.'); 
      ShranjenNaslov[IndNajnizjeZapSt] := 0; 
      IndNajnizjeZapSt := IndNajnizjeZapSt mod ShranjenNaslovM + 1; 
      NajvecjaZapSt := NajvecjaZapSt + 1; 
    end; {while}
  end; {IzpisiKarMores} 
  
begin {ObdelajZaporedjeNaslovov}
  ZapSt := 0; vObdelavi := 0; 
  IndNajnizjeZapSt := 1; NajvecjaZapSt := ShranjenNaslovM; 
  for j := 1 to ShranjenNaslovM do ShranjenNaslov[j] := 0; 
  for ImeRac := 10001 to 10000 + Koliko do begin
    ZapSt := ZapSt + 1; 
    while (vObdelavi >= StProcesorjev) or (ZapSt > NajvecjaZapSt) do 
      IzpisiKarMores; 
    j := Obdelaj(ImeRac); 
    PripadajocaZapSt[j] := ZapSt; vObdelavi := vObdelavi + 1; 
  end; {while}
  while vObdelavi > 0 do IzpisiKarMores; 
end; {ObdelajZaporedjeNaslovov} 

{-----------------------------------------------------------------------------}
{ Preostanek programa simulira dogajanje v omreju.                           }
{-----------------------------------------------------------------------------}

var Cas: integer;
    KajDela: array[1..StProcesorjev] of integer;
    DoKdaj: array[1..StProcesorjev] of integer;

procedure IzpisiStanje;
var i: integer;
begin
  Write('[T=', Cas:3, '] ');
  for i := 1 to StProcesorjev do 
    if KajDela[i] > 0 then Write(KajDela[i], '/', DoKdaj[i]:3, '  ')
    else Write('-----/---  ');
  WriteLn;
end; {IzpisiStanje}

function Obdelaj(Ime: Niz): integer;
var i, Trajanje: integer;
begin
  { Poiimo kak prost procesor. }
  for i := 1 to StProcesorjev do if KajDela[i] = 0 then begin 
    KajDela[i] := Ime;
    { Izmislimo si, kako dolgo bo trajala poizvedba. }
    case Random(10) of
    0: Trajanje := 91 + Random(10);  { 91..100 }
    1: Trajanje := 21 + Random(70);  { 21..90 }
    2: Trajanje := 1 + Random(5);    { 1..5 }
    else Trajanje := 6 + Random(14); { 6..20 }
    end; {case}
    DoKdaj[i] := Cas + Trajanje; Obdelaj := i; 
    Write('Obdelaj(', Ime:5, ') '); IzpisiStanje; 
    exit;
  end; {for, if}
  { e pridemo do sem, pomeni, da so vsi procesorji zasedeni. }
  Obdelaj := 0;
end; {Obdelaj}

function PoberiRezultat(var Naslov: Niz): integer;
var i, Proc, Kdaj: integer;
begin
  { Poglejmo, katera od poizvedb, ki so trenutno v
    teku, se bo najprej konala. }
  Kdaj := -1; Proc := 0;
  for i := 1 to StProcesorjev do if KajDela[i] > 0 then
    if (Kdaj < 0) or (Kdaj > DoKdaj[i]) then
      begin Proc := i; Kdaj := DoKdaj[i]; end;
  PoberiRezultat := Proc; if Proc = 0 then exit;
  { Prestavimo se v asovni trenutek, ko se prva poizvedba kona. }
  Naslov := KajDela[Proc]; KajDela[Proc] := 0; Cas := Kdaj;
  Write('Poberi(', Naslov:5, ')  '); IzpisiStanje;
end; {PoberiRezultat}

var i, j: integer;
begin {Naslovi}
  for i := 1 to StProcesorjev do KajDela[i] := 0; 
  Cas := 0;
  if ParamCount >= 1 then begin
    Val(ParamStr(1), i, j); WriteLn('tevilo naslovov: ', i); 
  end else begin
    Write('Vnesi tevilo naslovov: '); ReadLn(i);
  end; {if}
  Write('Zaetno stanje '); IzpisiStanje;
  ObdelajZaporedjeNaslovov(i);
end. {Naslovi}