program VpliviKopica; { RTK 2003.3.3 }
{ Reitev z dvojiko kopico.  asovna zahtevnost je v 
  najslabem primeru O(m lg n). }
const MaxN = 1000; MaxM = 200000; MaxVpliv = 1000000000;
type TabelaN = array[1..MaxN] of longint;
     TabelaM = array[1..MaxM] of longint;
var 
  T: text; i, u, v, m, n, StVKopici: longint; p, Kand: longint;
  Kdo, NaKoga1, Vpliv1, NaKoga, Vpliv, Kopica: ^TabelaM; 
  NaKoliko, Prvi, NajVpliv: ^TabelaN; 
  KjeVKopici: ^TabelaN; { 0, e ni v kopici }

  { Predpostavi, da se element v trenutno nahaja v korenu 
    kopice (eprav ni nujno, da je Kopica^[1] res enako v).  
    S pogrezanjem poskrbi, da bo priel v na takno mesto, kjer 
    bo njegova vrednost NajVpliv^[v] vsaj tolikna kot pri 
    njegovih dveh otrocih. }
  procedure Pogrezni(v: longint);
  var i, ci, c, c2: longint;
  begin
    i := 1;
    while 2 * i <= StVKopici do begin
      { Element na indeksu i ima otroka na indeksih 
        ci = 2 * i in 2 * i + 1. }
      ci := 2 * i; c := Kopica^[ci];
      if ci < StVKopici then begin 
        { Element i ima e drugega otroka; ci naj kae na
          tistega otroka, ki ima vejo vrednost NajVpliv. }
        c2 := Kopica^[ci + 1];
        if NajVpliv^[c2] > NajVpliv^[c] then 
          begin c := c2; Inc(ci); end; 
      end; {if}
      { e ima v vsaj toliken NajVpliv kot otroka, 
        lahko s pogrezanjem nehamo. }
      if NajVpliv^[v] >= NajVpliv^[c] then break;
      { Zamenjajmo element v na indeksu i z otrokom c na
        indeksu ci.  v-ja e ne bomo vpisovali v celico
        Kopica^[ci], saj ga bo mogoe treba e naprej
        pogrezati.  Zato ga bomo vpisali ele na koncu. }
      KjeVKopici^[c] := i; Kopica^[i] := c; i := ci;
    end; {while}
    { Pogrezanje se je ustavilo na indeksu i, 
      zato zdaj tja vpiimo element v. }
    KjeVKopici^[v] := i; Kopica^[i] := v;
  end; {Pogrezni}
  
  { Vrednost NajVpliv^[v] povea na NoviVpliv in po 
    potrebi dvigne element v gor po kopici, da kopica 
    ohrani zahtevane lastnosti. }
  procedure Dvigni(v, NoviVpliv: longint);
  var pi, p, i: longint;
  begin
    NajVpliv^[v] := NoviVpliv; i := KjeVKopici^[v]; 
    while i > 1 do begin
      { Element v je trenutno na indeksu i.  
        Njegov oe je na indeksu pi = i div 2. }
      pi := i div 2; p := Kopica^[pi];
      { e ima oe dovolj veliko vrednost NajVpliv, 
        lahko z dvigovanjem prenehamo. }
      if NajVpliv^[p] >= NoviVpliv then break;
      { Zamenjajmo element v na indeksu i z njegovim oetom p 
        na indeksu pi.  Ker bo treba v mogoe dvigovati e 
        naprej, ga zaenkrat e ne bomo vpisali v celico pi. }
      KjeVKopici^[p] := i; Kopica^[i] := p; i := pi;
    end; {while}
    { Konec dvigovanja -- element v je pristal na indeksu i. }
    KjeVKopici^[v] := i; Kopica^[i] := v;
  end; {Dvigni}
  
begin {VpliviKopica}
  if ParamCount >= 1 then Assign(T, ParamStr(1)) else Assign(T, 'vplivi.in');
  Reset(T); ReadLn(T, n); ReadLn(T, m);
  
  { Preberimo vse povezave (u, v) v tabeli Kdo in NaKoga1.
    V Vpliv1 shranimo vplive povezav, v NaKoliko pa za
    vsakega inkvizitorja zapiemo, na koliko drugih vpliva. }
  New(Kdo); New(NaKoga1); New(Vpliv1); New(NaKoliko); 
  for u := 1 to n do NaKoliko^[u] := 0;
  for i := 1 to m do begin
    ReadLn(T, u, v, p); Inc(NaKoliko^[u]);
    Kdo^[i] := u; NaKoga1^[i] := v; Vpliv1^[i] := p; 
  end; {for}
  Close(T); New(Prvi); New(NaKoga); New(Vpliv);
  
  { Povezave uredimo tako, da bomo imeli za vsakega inkvizitorja 
    pri roki seznam vseh, na katere vpliva: za inkvizitorja u
    so to inkvizitorji NaKoga^[Prvi^[u]..Prvi^[u] + NaKoliko^[u] - 1]. }
  Prvi^[1] := 1; for u := 2 to n do Prvi^[u] := Prvi^[u-1] + NaKoliko^[u-1];
  for u := 1 to n do NaKoliko^[u] := 0;
  for i := 1 to m do begin
    u := Kdo^[i]; 
    NaKoga^[Prvi^[u] + NaKoliko^[u]] := NaKoga1^[i];
    Vpliv^[Prvi^[u] + NaKoliko^[u]] := Vpliv1^[i];
    Inc(NaKoliko^[u]);
  end; {for}
  Dispose(Kdo); Dispose(NaKoga1); Dispose(Vpliv1); New(NajVpliv);

  { Glavni del naega postopka.  V kopici (tabela Kopica) je
    StVKopici tok.  Tabela KjeVKopici pove, v kateri celici tabele
    Kopica se nahaja neka toka (0 pomeni, da te toke ni v kopici). 
    Na zaetku je v kopici samo veliki inkvizitor (toka 1). }
  NajVpliv^[1] := MaxVpliv + 1; for u := 2 to n do NajVpliv^[u] := -1;
  New(Kopica); New(KjeVKopici); StVKopici := 1;
  for u := 2 to n do KjeVKopici^[u] := 0; 
  Kopica^[1] := 1; KjeVKopici^[1] := 1;
  while StVKopici > 0 do begin
    { Vzemimo iz kopice toko z najvejo vrednostjo NajVpliv.
      To je tista, ki je trenutno v korenu (Kopica^[1]); nato
      kliemo Pogrezni, da bo v to celico premestil element 
      s konca kopice in ga nato pogreznil do primerne globine.
      tevilo elementov v kopici se pri tem seveda zmanja za 1. }
    u := Kopica^[1]; KjeVKopici^[u] := -1; 
    Dec(StVKopici); Pogrezni(Kopica^[StVKopici + 1]);
    { Na koga vse lahko u vpliva? }
    for i := 1 to NaKoliko^[u] do begin 
      v := NaKoga^[Prvi^[u] + i - 1]; p := Vpliv^[Prvi^[u] + i - 1];
      if p < NajVpliv^[u] then Kand := p else Kand := NajVpliv^[u];
      if Kand > NajVpliv^[v] then begin
        { Odkrili smo novo najvplivnejo pot do toke v.
          e v-ja e ni v kopici, ga moramo tja dodati, drugae
          pa moramo vsaj preveriti, e ga je treba kaj dvigniti,
          saj se mu je poveala vrednost NajVpliv^[v]. }
        if KjeVKopici^[v] <= 0 then 
          { Ni ga e v kopici -- dodajmo ga kar na konec, saj bo
            e podprogram Dvigni, ki ga kliemo spodaj, poskrbel,
            da se bo dvignil na primerno mesto v kopici. }
          begin Inc(StVKopici); KjeVKopici^[v] := StVKopici; end;
        Dvigni(v, Kand);
      end; {if}
    end; {for}
  end; {while}
  Dispose(Kopica); Dispose(KjeVKopici); Dispose(Vpliv); 
  Dispose(NaKoga); Dispose(NaKoliko); Dispose(Prvi);
  
  { Izpis rezultatov. }
  if ParamCount >= 2 then Assign(T, ParamStr(2)) 
  else Assign(T, 'vplivi.out');
  Rewrite(T); for u := 2 to n do WriteLn(T, NajVpliv^[u]);
  Close(T); Dispose(NajVpliv);
end. {VpliviKopica}
