program VpliviOOP; { RTK 2003.3.3 }
{ Ta razliica reitve vsebuje implementacije ve 
  podatkovnih struktur za hranjenje mnoice tok:
  sklad, vrsto in kopico.  Vsaki pripada po en razred,
  izpeljan iz abstraktnega razreda TMnozica.
  Tip podatkovne strukture lahko izberemo s tretjim
  parametrom v ukazni vrstici: 
    <vhodna-datoteka> <izhodna-datoteka> <tip>
  Pri tem je <tip> lahko 'V', 'S' ali 'K'.  e je ta
  parameter prisoten, bo program tudi izpisal tevilo
  izvajanj zunanje in notranje zanke v glavnem delu algoritma. }
const MaxN = 1000; MaxM = 200000; MaxVpliv = 1000000000;
type TabelaN = packed array[1..MaxN] of longint;
     TabelaM = packed array[1..MaxM] of longint;
     TabelaB = packed array[1..MaxN] of boolean;
type
  { Abstraktni razred (oz. interface), igar potomci
    bodo znali hraniti mnoico tok grafa. }
  PMnozica = ^TMnozica;
  TMnozica = object
    constructor Init;
    destructor Done; virtual;
    { Pove, ali je mnoica prazna. }
    function Prazna: boolean; virtual; abstract;
    { Iz mnoice vzame poljubno toko in jo vrne kot rezultat funkcije. }
    function Vzemi: longint; virtual; abstract;
    { Doda dano toko v mnoico.  Mogoe je e od prej v mnoici,
      pa se ji je samo poveala vrednost NajVpliv^[v]. }
    procedure Dodaj(v: longint); virtual; abstract;
  end; {TMnozica}

constructor TMnozica.Init;
begin
end; {TMnozica.Done}

destructor TMnozica.Done;
begin
end; {TMnozica.Done}

var
  { Naslednji globalni spremenljivki bosta prili prav
    vsem potomcem razreda TMnozica. }
  n: longint; { tevilo tok grafa }
  NajVpliv: ^TabelaN; { vplivi najvplivnejih doslej znanih poti od 1 do ostalih tok } 

{-- Sklad --------------------------------------------------------------------}

type
  PSklad = ^TSklad;
  TSklad = object(TMnozica)
    Sklad: TabelaN;    { vrh sklada je celica Sklad^[Dolzina] }
    NaSkladu: TabelaB; { za vsako toko pove, ali je na skladu }
    Dolzina: integer;
    constructor Init;
    function Prazna: boolean; virtual; 
    function Vzemi: longint; virtual; 
    procedure Dodaj(v: longint); virtual; 
  end; {TSklad}

constructor TSklad.Init; 
var i: integer;
begin 
  Dolzina := 0; 
  for i := 1 to n do NaSkladu[i] := false;
end; {TSklad.Init}

function TSklad.Prazna: boolean;
begin
  Prazna := Dolzina = 0;
end; {TSklad.Prazna}

function TSklad.Vzemi: longint; 
var v: longint;
begin
  v := Sklad[Dolzina]; Vzemi := v;
  NaSkladu[v] := false; Dec(Dolzina);
end; {TSklad.Vzemi}

procedure TSklad.Dodaj(v: longint);
begin
  if not NaSkladu[v] then begin
    Inc(Dolzina); Sklad[Dolzina] := v;
    NaSkladu[v] := true;
  end; {if}
end; {TSklad.Dodaj}

{-- Vrsta --------------------------------------------------------------------}
     
type
  PVrsta = ^TVrsta;
  TVrsta = object(TMnozica)
    Vrsta: TabelaN;    { uporabljene so celice od Glava do Glava + Dolzina - 1 }
    VVrsti: TabelaB;   { za vsako toko pove, ali je v vrsti }
    Glava, Dolzina: integer;
    constructor Init;
    function Prazna: boolean; virtual; 
    function Vzemi: longint; virtual; 
    procedure Dodaj(v: longint); virtual; 
  end; {TVrsta}

constructor TVrsta.Init; 
var i: integer;
begin 
  Glava := 1; Dolzina := 0; 
  for i := 1 to n do VVrsti[i] := false;
end; {TVrsta.Init}

function TVrsta.Prazna: boolean;
begin
  Prazna := Dolzina = 0;
end; {TVrsta.Prazna}

function TVrsta.Vzemi: longint; 
var v: longint;
begin
  v := Vrsta[Glava]; Vzemi := v;
  VVrsti[v] := false; Dec(Dolzina);
  Glava := (Glava mod n) + 1;
end; {TVrsta.Vzemi}

procedure TVrsta.Dodaj(v: longint);
begin
  if not VVrsti[v] then begin
    Vrsta[((Glava + Dolzina + n - 1) mod n) + 1] := v;
    Inc(Dolzina); VVrsti[v] := true;
  end; {if}
end; {TVrsta.Dodaj}
     
{-- Kopica -------------------------------------------------------------------}

type
  PKopica = ^TKopica;
  TKopica = object(TMnozica)
    Kopica: TabelaN;      { uporabljene so celice 1..StVKopici }
    KjeVKopici: TabelaN;  { 0, e ni v kopici }
    StVKopici: integer;
    constructor Init;
    function Prazna: boolean; virtual; 
    function Vzemi: longint; virtual; 
    procedure Dodaj(v: longint); virtual; 
  end; {TKopica}

constructor TKopica.Init; 
var i: integer;
begin 
  StVKopici := 0; 
  for i := 1 to n do KjeVKopici[i] := 0;
end; {TKopica.Init}

function TKopica.Prazna: boolean;
begin
  Prazna := StVKopici = 0;
end; {TKopica.Prazna}

function TKopica.Vzemi: longint; 
var v, i, ci, c, c2: longint;
begin
  v := Kopica[1]; Vzemi := v; 
  KjeVKopici[v] := 0;
  { Prestavimo element s konca kopice v koren. }
  v := Kopica[StVKopici]; Dec(StVKopici);
  i := 1;
  while 2 * i <= StVKopici do begin
    { Element na indeksu i ima otroka na indeksih 
      ci = 2 * i in 2 * i + 1. }
    ci := 2 * i; c := Kopica[ci];
    if ci < StVKopici then begin 
      { Element i ima e drugega otroka; ci naj kae na
        tistega otroka, ki ima vejo vrednost NajVpliv. }
      c2 := Kopica[ci + 1];
      if NajVpliv^[c2] > NajVpliv^[c] then 
        begin c := c2; Inc(ci); end; 
    end; {if}
    { e ima v vsaj toliken NajVpliv kot otroka, 
      lahko s pogrezanjem nehamo. }
    if NajVpliv^[v] >= NajVpliv^[c] then break;
    { Zamenjajmo element v na indeksu i z otrokom c na
      indeksu ci.  v-ja e ne bomo vpisovali v celico
      Kopica^[ci], saj ga bo mogoe treba e naprej
      pogrezati.  Zato ga bomo vpisali ele na koncu. }
    KjeVKopici[c] := i; Kopica[i] := c; i := ci;
  end; {while}
  { Pogrezanje se je ustavilo na indeksu i, 
    zato zdaj tja vpiimo element v. }
  KjeVKopici[v] := i; Kopica[i] := v;
end; {TKopica.Vzemi}

procedure TKopica.Dodaj(v: longint);
var pi, p, i: longint;
begin
  if KjeVKopici[v] = 0 then begin
    { Toke v e ni v kopici -- dodajmo jo na konec. }
    Inc(StVKopici); 
    KjeVKopici[v] := StVKopici;
  end; {if}
  { Dvignimo element v na primerno mesto v kopici. }
  i := KjeVKopici[v]; 
  while i > 1 do begin
    { Element v je trenutno na indeksu i.  
      Njegov oe je na indeksu pi = i div 2. }
    pi := i div 2; p := Kopica[pi];
    { e ima oe dovolj veliko vrednost NajVpliv, 
      lahko z dvigovanjem prenehamo. }
    if NajVpliv^[p] >= NajVpliv^[v] then break;
    { Zamenjajmo element v na indeksu i z njegovim oetom p 
      na indeksu pi.  Ker bo treba v mogoe dvigovati e 
      naprej, ga zaenkrat e ne bomo vpisali v celico pi. }
    KjeVKopici[p] := i; Kopica[i] := p; i := pi;
  end; {while}
  { Konec dvigovanja -- element v je pristal na indeksu i. }
  KjeVKopici[v] := i; Kopica[i] := v;
end; {TKopica.Dodaj}

{-- Glavni del programa ------------------------------------------------------}
     
var 
  T: text; i, u, v, m: longint; p, Kand: longint;
  Kdo, NaKoga1, Vpliv1, NaKoga, Vpliv: ^TabelaM; 
  NaKoliko, Prvi: ^TabelaN; 
  Q: PMnozica; StZunanje, StNotranje: int64;
begin {VpliviOOP}
  if ParamCount >= 1 then Assign(T, ParamStr(1)) else Assign(T, 'vplivi.in');
  Reset(T); ReadLn(T, n); ReadLn(T, m);
  
  { Preberimo vse povezave (u, v) v tabeli Kdo in NaKoga1.
    V Vpliv1 shranimo vplive povezav, v NaKoliko pa za
    vsakega inkvizitorja zapiemo, na koliko drugih vpliva. }
  New(Kdo); New(NaKoga1); New(Vpliv1); New(NaKoliko); 
  for u := 1 to n do NaKoliko^[u] := 0;
  for i := 1 to m do begin
    ReadLn(T, u, v, p); Inc(NaKoliko^[u]);
    Kdo^[i] := u; NaKoga1^[i] := v; Vpliv1^[i] := p; 
  end; {for}
  Close(T); New(Prvi); New(NaKoga); New(Vpliv);
  
  { Povezave uredimo tako, da bomo imeli za vsakega inkvizitorja 
    pri roki seznam vseh, na katere vpliva: za inkvizitorja u
    so to inkvizitorji NaKoga^[Prvi^[u]..Prvi^[u] + NaKoliko^[u] - 1]. }
  Prvi^[1] := 1; for u := 2 to n do Prvi^[u] := Prvi^[u-1] + NaKoliko^[u-1];
  for u := 1 to n do NaKoliko^[u] := 0;
  for i := 1 to m do begin
    u := Kdo^[i]; 
    NaKoga^[Prvi^[u] + NaKoliko^[u]] := NaKoga1^[i];
    Vpliv^[Prvi^[u] + NaKoliko^[u]] := Vpliv1^[i];
    Inc(NaKoliko^[u]);
  end; {for}
  Dispose(Kdo); Dispose(NaKoga1); Dispose(Vpliv1); 

  { Pripravimo strukturo za hranjenje mnoice tok. }
  if ParamCount >= 3 then
    case UpCase(ParamStr(3)[1]) of
    'V': begin Q := New(PVrsta, Init); Write('vrsta'); end;
    'S': begin Q := New(PSklad, Init); Write('sklad'); end;
    else begin Q := New(PKopica, Init); Write('kopica'); end;
    end
  else Q := New(PKopica, Init);

  { Glavni del naega postopka.  
    Na zaetku je v mnoici Q samo veliki inkvizitor (toka 1). }
  New(NajVpliv); NajVpliv^[1] := MaxVpliv + 1; 
  for u := 2 to n do NajVpliv^[u] := -1;
  Q^.Dodaj(1); StZunanje := 0; StNotranje := 0;
  while not Q^.Prazna do begin
    { Vzemimo iz kopice toko z najvejo vrednostjo NajVpliv.
      To je tista, ki je trenutno v korenu (Kopica^[1]); nato
      kliemo Pogrezni, da bo v to celico premestil element 
      s konca kopice in ga nato pogreznil do primerne globine.
      tevilo elementov v kopici se pri tem seveda zmanja za 1. }
    u := Q^.Vzemi; 
    Inc(StZunanje); Inc(StNotranje, NaKoliko^[u]);
    { Na koga vse lahko u vpliva? }
    for i := 1 to NaKoliko^[u] do begin 
      v := NaKoga^[Prvi^[u] + i - 1]; p := Vpliv^[Prvi^[u] + i - 1];
      if p < NajVpliv^[u] then Kand := p else Kand := NajVpliv^[u];
      if Kand > NajVpliv^[v] then begin
        { Odkrili smo novo najvplivnejo pot do toke v.
          Dodajmo ga v mnoico Q. }
        NajVpliv^[v] := Kand;
        Q^.Dodaj(v);
      end; {if}
    end; {for}
  end; {while}
  Dispose(Vpliv); Dispose(NaKoga); Dispose(NaKoliko); Dispose(Prvi);
  Dispose(Q, Done);
  
  { Izpis rezultatov. }
  if ParamCount >= 2 then Assign(T, ParamStr(2)) 
  else Assign(T, 'vplivi.out');
  Rewrite(T); for u := 2 to n do WriteLn(T, NajVpliv^[u]);
  Close(T); Dispose(NajVpliv);
  if ParamCount >= 3 then WriteLn(' ', StZunanje, ' ', StNotranje);
end. {VpliviOOP}
