program NarascajocaStevila; { RTK 2004.2.1 }
const MaxN = 100;
var N: integer;
    A: packed array [1..MaxN] of integer;

{ Program s standardnega vhoda prebere najprej N
  in nato e tevila A[1], ..., A[N]. }

{ Preprosta reitev s tabelo booleanov, ki povedo, katera
  tevila smo e izpisali in katerih ne.  Lahko bi uporabili
  tudi pascalske tipe za mnoice -- Neizpisano bi bila
  mnoica tipa 'set of 1..N'.  V obeh primerih je slabost te
  reitve ta, da v najslabem primeru tee O(N^2) asa. }
procedure Resitev1;
var 
  Neizpisano: packed array [1..MaxN] of boolean;
  i, x: integer;
begin
  for x := 1 to N do Neizpisano[x] := true;
  for i := 1 to N do begin
    x := A[i];
    while not Neizpisano[x] do 
      begin x := x + 1; if x > N then x := 1 end;
    Write(x, ' ');
    Neizpisano[x] := false;
  end; {for i}
  WriteLn;
end; {Resitev1}

{ Izboljana oblika prve reitve.  Namesto ene same tabele
  (kot Neizpisano v prvi reitvi) ima e ve manjih tabel,
  ki povedo za skupine po dveh, tirih, osmih itd. tevil,
  e so bila e vsa ta tevila izpisana ali pa je vsaj eno
  od njih e neizpisano.  Takne tabele lahko uporabimo, da
  veliko hitreje odkrijemo naslednje neizpisano tevilo. 
  asovna zahtevnost je le e O(N log N). }
procedure Resitev2;
type
  TabelaP = ^TabelaT;
  TabelaT = packed array [0..MaxN - 1] of boolean;
var
  { T[i]^[j] = true natanko tedaj, e je vsaj eno od tevil
    z intervala (i shl j)..((i + 1) shl j)-1 e neizpisano. }
  T: array [0..31] of TabelaP;
  DT: array [0..31] of integer;
  nTabel, i, j, x: integer;
begin

  { Pripravimo tabele T[0..nTabel-1].  T[0] ima N elementov,
    T[1] ima ceil(N/2) elementov, T[2] ima ceil(N/4) elementov
    in tako naprej.  Zadnja ima en sam element.  Na zaetku
    so vsi elementi true, ker nismo izpisali e nobenega tevila. 
    Zaradi lajega raunanja z indeksi bomo uporabljali v teh
    tabelah tevila od 0 do N-1 namesto od 1 do N. }
  i := N; nTabel := 0;
  while true do begin
    DT[nTabel] := i;
    GetMem(T[nTabel], DT[nTabel] * SizeOf(boolean));
    for j := 0 to DT[nTabel] - 1 do T[nTabel]^[j] := true;
    nTabel := nTabel + 1; 
    if i = 1 then break else i := (i + 1) div 2;
  end; {while}
  
  for i := 1 to N do begin
    x := A[i] - 1; j := 0;
    while not T[j]^[x shr j] do begin
      { Na tem mestu velja, da so bila tevila v skupini 2^j tevil,
        ki vsebuje tudi x, e vsa izpisana, torej se lahko premaknemo 
        z x na konec te skupine.  Poleg tega ta skupina ne vsebuje
        nobenih tevil, manjih ali enakih A[i] - 1.  Poleg tega
        je x vekratnik tevila 2^(j-1). }
      if j > 0 then Assert(x and ((1 shl (j - 1)) - 1) = 0);
      Assert((x shr j) shl j >= A[i] - 1);
      x := ((x shr j) + 1) shl j;
      if x >= N then 
        { Prili smo do konca tabele, torej bo treba zaeti spet na
          zaetku in izpisati najmanje e neizpisano tevilo. }
        begin x := 0; j := nTabel; break end;
      j := j + 1;
    end; {while}
    if j > 0 then begin
      Assert(x and ((1 shl (j - 1)) - 1) = 0);
      { Zaradi zgoraj omenjene invariante lahko postavimo x kar na
        zaetek skupine 2^j tevil, ki ji x pripada. }
      x := x and not (1 shl (j - 1));
    end; {if}

    { Zdaj vemo, da je med tevili od x do x + 2^j - 1 
      vsaj eno e neizpisano.  Poiskati moramo najmanje tako tevilo
      in ga izpisati. }
    while j > 0 do begin
      { Skupino 2^j tevil z zaetkom pri x (ki je, mimogrede,
        vekratnik tevila 2^j) razdelimo na dve pol manji skupini,
        torej s po 2^(j-1) tevili, ena se zane pri x, druga
        pa pri x + 2^(j-1).  Ker iemo najmanje e neizpisano
        tevilo, moramo nadaljevati v prvi podskupini, razen e so bila
        tam e vsa tevila izpisana. } 
      Assert(x and ((1 shl j) - 1) = 0);
      if not T[j - 1]^[x shr (j - 1)] then begin
        x := x + (1 shl (j - 1)); Assert(x < N);
        Assert(T[j - 1]^[x shr (j - 1)]);
      end; {if}
      j := j - 1;
    end; {while}
    { Zdaj smo nali tevilo x, ki ga je treba izpisati.  No, pravzaprav
      bo treba izpisati x + 1, ker naloga govori o tevilih 1..N, mi
      pa uporabljamo 0..(N-1). }
    Write(x + 1, ' ');

    { Ker je bilo tevilo x zdaj izpisano, moramo ustrezno popraviti 
      tabele.  Za zaetek si zapiimo, da x sam ni ve neizpisan. }
    Assert(j = 0); Assert(T[j]^[x shr j]);
    T[j]^[x shr j] := false;
    while j + 1 < nTabel do begin
      { Skupina 2^(j+1) tevil vsebuje kakno neizpisano tevilo natanko
        tedaj, ko vsaj ena od njenih dveh podskupin s po 2^j tevili
        vsebuje kakno neizpisano tevilo.  Poglejmo torej, kako je s
        tisto skupino 2^(j+1) tevil, ki ji pripada tudi x. }
      T[j + 1]^[x shr (j + 1)] := 
        T[j]^[x shr j] or T[j]^[(x shr j) xor 1];
      { Na zaetku so bili vsi elementi naih tabel T enaki true.
        e je gornji stavek vrednost true ohranil, se torej ni ni
        spremenilo in lahko nehamo. }
      if T[j + 1]^[x shr (j + 1)] then break;
      { Drugae pa bo ta sprememba mogoe povzroila tudi spremembe
        v vije leeih tabelah in moramo s to zanko nadaljevati. }
      j := j + 1;
    end; {while}
    
    { Za bolje razumevanje delovanja tega postopka (ali pa za laje
      odpravljanje napak) je vasih koristno izpisati tabele T. }
    if false then 
      for j := 0 to nTabel - 1 do begin
        for x := 0 to DT[j] - 1 do 
          if T[j]^[x] then Write('#') else Write('.');
       WriteLn;
      end; {for j}
  end; {for i}

  { Pospravimo za sabo. }
  for j := 0 to nTabel - 1 do
    FreeMem(T[j], DT[j] * SizeOf(boolean));
  WriteLn;
end; {Resitev2}

{ Glavni blok programa. }
 
var i: integer;
begin {NarascajocaStevila}
  Read(N);
  for i := 1 to N do begin 
    Read(A[i]);
    Assert(1 <= A[i]); Assert(A[i] <= N);
  end; {for i}
  
  Resitev1;
  Resitev2;
end. {NarascajocaStevila}
