program Otoki; { RTK 2004.3.1 }

{ e je definiran simbol QUICKSORT, bo program celice najprej
  uredil po viini (s quicksortom) in jih potem pregledoval v
  tem vrstnem redu.  e ta simbol ni definiran, program vsaki
  pregleda vse celice, da ugotovi, katera bo naslednja pogledala
  iz vode. }
{$DEFINE QUICKSORT}
{ e je definiran simbol HITRO_ZLIVANJE, bo program zlil dva
  otoka tako, da bo el po vseh celicah manjega otoka in jih
  popravil, da pripadajo vejemu otoku.  e ta simbol ni
  definiran, bo el program pri zlivanju po vseh celicah 
  cele mree. 
    e sta definirana tako QUICKSORT kot HITRO_ZLIVANJE, je
  asovna zahtevnost programa O(n log n) za mreo n celic,
  sicer pa O(n^2). }
{$DEFINE HITRO_ZLIVANJE}
{ e je definiran simbol IZPISUJ, bo program med delom izpisoval
  podatke o stanju mree (zemljevid otokov, ki so e nad vodo). }
{$DEFINE ne_IZPISUJ}
{ e je definiran simbol KROGLA, bo program uporabljal malo
  drugano definicijo sosednosti, ki je primerna za okrogel
  planet namesto za ploatega. }
{$DEFINE ne_KROGLA}

const 
  MaxW = 100; MaxH = 100;
  {MaxW = 10800; MaxH = 5400;} { Za mreo ETOPO2.doc, glej spodaj. }
  DX: array [1..4] of integer = (-1, 1, 0, 0);
  DY: array [1..4] of integer = (0, 0, -1, 1);
type TabelaT = packed array [1..MaxH * MaxW] of integer;
var 
  W, H: integer;
  V, Otok: array [1..MaxH, 1..MaxW] of integer;
  
  function Soseda(NX, NY, i: integer; var SX, SY: integer): boolean;
  begin
    SX := NX + DX[i]; SY := NY + DY[i];
    {$IFDEF KROGLA}
    if SY = -1 then begin SY := 0; SX := SX + (W div 2) end;
    if SY = H then begin SY := H - 1; SX := SX + (W div 2) end;
    if SX < 0 then SX := SX + W
    else if SX >= W then SX := SX - W;
    Soseda := true;
    {$ELSE}
    Soseda := (SX >= 1) and (SY >= 1) and (SX <= W) and (SY <= H);
    {$ENDIF}
  end; {Soseda}

  { Na http://www.ngdc.noaa.gov/mgg/global/relief/ETOPO2/ETOPO2.dos
    so podatki za celo Zemljo z loljivostjo dveh kotnih minut.
    To je datoteka s (360 * 30) x (180 * 30) predznaenimi 16-bitnimi
    celimi tevili.  Podatki so v metrih, z intervala -10654..8593. }
  procedure BeriBinarno(Ime: string);
  const Sirina = 360 * 30;
  type VrsticaT = packed array [1..Sirina] of SmallInt; { predznaena 16-bitna tevila }
  var F: file; x, y, n, MinV, MaxV: integer; Vrstica: VrsticaT;
  begin
    W := Sirina; H := 180 * 30;
    Assign(F, Ime); Reset(F, 1);
    for y := 1 to H do begin
      BlockRead(F, Vrstica, SizeOf(Vrstica), n); 
      Assert(n = SizeOf(Vrstica));
      for x := 1 to W do begin
        V[y, x] := integer(Vrstica[x]);
        if (y = 1) and (x = 1) then begin MinV := V[y, x]; MaxV := V[y, x] end
        else begin
          if V[y, x] < MinV then MinV := V[y, x];
          if V[y, x] > MaxV then MaxV := V[y, x];
        end; {if}
      end; {for}
    end; {for y}
    Close(F);
    WriteLn('Razpon visin: ', MinV, '..', MaxV);
  end; {BeriVhod}
  
  procedure BeriTxt(Ime: string);
  var T: text; x, y: integer;
  begin
    Assign(T, Ime);
    Reset(T); ReadLn(T, H, W);
    for y := 1 to H do begin
      for x := 1 to W do Read(T, V[y, x]);
      ReadLn(T);
    end; {for y};
    Close(T);
  end; {Beri}
  
{$IFDEF QUICKSORT}
{ V tej tabeli je celica (X, Y) predstavljena s tevilom (Y - 1) * W + (X - 1). 
  Ta tevila so urejena po padajoi viini celic. }
var VrstniRed: ^TabelaT;

  procedure UrediPoVisini;
  
    procedure QuickSort(L, R: integer); { Uredi elemente [L..R]. }
    var M, i, j, T: integer;
    begin
      while L < R do begin
        M := VrstniRed^[(L + R) div 2];
        M := V[M div W + 1, M mod W + 1];
        i := L; j := R;
        while i < j do begin
          while V[VrstniRed^[i] div W + 1, VrstniRed^[i] mod W + 1] < M do i := i + 1;
          while V[VrstniRed^[j] div W + 1, VrstniRed^[j] mod W + 1] > M do j := j - 1;
          if i <= j then begin
            T := VrstniRed^[i]; VrstniRed^[i] := VrstniRed^[j]; VrstniRed^[j] := T;
            i := i + 1; j := j - 1;
          end; {if}
        end; {while}
        if j - L < R - i then begin QuickSort(L, j); L := i end
        else begin QuickSort(i, R); R := j end;
      end; {while}
    end; {QuickSort}
  
  var i: integer;
  begin {UrediPoVisini}
    for i := 1 to W * H do VrstniRed^[i] := i - 1;
    QuickSort(1, W * H);
  end; {UrediPoVisini}
{$ENDIF}

{$IFDEF HITRO_ZLIVANJE}
var VelOtoka, Vrsta: ^TabelaT;

  { Z iskanjem v irino obiimo vse celice manjega izmed danih
    dveh otokov in jih popravimo, da bodo pripadale vejemu otoku. }
  procedure ZlijOtoka(NX, NY, SX, SY: integer);
  var x, y, i, xx, yy, StariOtok, NoviOtok, Glava, Rep: integer;
  begin
    if VelOtoka^[Otok[NY, NX]] > VelOtoka^[Otok[SY, SX]] then begin 
      StariOtok := Otok[SY, SX]; NoviOtok := Otok[NY, NX]; x := SX; y := SY;
    end else begin
      NoviOtok := Otok[SY, SX]; StariOtok := Otok[NY, NX]; x := NX; y := NY;
    end; {if}
    Glava := 1; Rep := 2; Vrsta^[Glava] := (y - 1) * W + (x - 1);
    Otok[y, x] := NoviOtok; 
    while Glava < Rep do begin
      x := Vrsta^[Glava] mod W + 1; y := Vrsta^[Glava] div W + 1;
      Glava := Glava + 1; 
      for i := 1 to 4 do if Soseda(x, y, i, xx, yy) then 
        if Otok[yy, xx] = StariOtok then begin 
          Vrsta^[Rep] := (yy - 1) * W + (xx - 1); 
          Rep := Rep + 1; Otok[yy, xx] := NoviOtok; 
        end; {if}
    end; {while}
    Assert(VelOtoka^[StariOtok] = Rep - 1);
    VelOtoka^[NoviOtok] := VelOtoka^[NoviOtok] + VelOtoka^[StariOtok];
    VelOtoka^[StariOtok] := -1;
  end; {if}

{$ELSE}

  procedure ZlijOtoka(NX, NY, SX, SY: integer);
  var x, y, NO: integer;
  begin
    NO := Otok[NY, NX];
    for y := 1 to H do for x := 1 to W do 
      if Otok[y, x] = NO then Otok[y, x] := Otok[SY, SX];
  end; {if}
  
{$ENDIF}
  
var  
  i, x, y, G, NovaG, NajG, NX, NY, SX, SY: integer;
  StOtokov, MaxStOtokov, StKopnih: integer;
  T: text; Ime: string;
begin
  { Preberimo vhodno datoteko. }
  if ParamCount >= 1 then Ime := ParamStr(1) else Ime := 'otoki.in';
  if Pos('ETOPO', Ime) > 0 then BeriBinarno(Ime)
  else BeriTxt(Ime);
  
  { Na zaetku so vse celice pod vodo.  Postavimo G na nekaj velikega. }
  G := V[1, 1];
  for y := 1 to H do for x := 1 to W do begin
    if G < V[y, x] then G := V[y, x];
    Otok[y, x] := 0; 
  end; {for y}
  G := G + 1; NajG := G;
  StOtokov := 0; MaxStOtokov := 1; 
  StKopnih := W * H;
  
{$IFDEF QUICKSORT}
  GetMem(VrstniRed, SizeOf(integer) * W * H);
  {WriteLn('Urejam...');}
  UrediPoVisini;
  {WriteLn('Konec urejanja.');}
{$ENDIF}
{$IFDEF HITRO_ZLIVANJE}
  GetMem(VelOtoka, SizeOf(integer) * W * H);
  GetMem(Vrsta, SizeOf(integer) * W * H);
{$ENDIF}

  while StKopnih > 0 do begin
  
    { Zniajmo gladino. }
{$IFDEF QUICKSORT}
    NX := VrstniRed^[StKopnih] mod W + 1;
    NY := VrstniRed^[StKopnih] div W + 1;
    NovaG := V[NY, NX];
{$ELSE}
    NovaG := -1;
    for y := 1 to H do for x := 1 to W do 
      if (Otok[y, x] = 0) and (V[y, x] > NovaG) then 
        begin NovaG := V[y, x]; NX := x; NY := y end;
{$ENDIF}
    Assert(NovaG <= G);
    
    if NovaG < G then begin
      if StOtokov > MaxStOtokov then 
        begin MaxStOtokov := StOtokov; NajG := G end;
{$IFDEF IZPISUJ}
      WriteLn('Gladina: ', G, ', tevilo otokov: ', StOtokov);
      for y := 1 to H do begin
        for x := 1 to W do begin
          if Otok[y, x] = 0 then Write('.')
          else Write(Chr(Ord('A') + (Otok[y, x] - 1) mod (Ord('Z') - Ord('A') + 1)));
        end; {for x}
        WriteLn; 
      end; {for y}
      WriteLn;
{$ENDIF}
    end; {if}
      
    G := NovaG;
    { Celica (NX, NY) je pogledala iz vode. }
    Otok[NY, NX] := NY * W + NX;
    StOtokov := StOtokov + 1; StKopnih := StKopnih - 1;
{$IFDEF HITRO_ZLIVANJE}
    VelOtoka^[Otok[NY, NX]] := 1;
{$ENDIF}
    { e ima kaj kopnih sosed, se otoki zdruujejo. }
    for i := 1 to 4 do if Soseda(NX, NY, i, SX, SY) then 
      if (Otok[SY, SX] <> 0) and (Otok[NY, NX] <> Otok[SY, SX]) then begin
        { Soseda (SX, SY) je kopna in ne pripada istemu otoku 
          kot (NX, NY), zato ta dva otoka zdruimo. }
        ZlijOtoka(NX, NY, SX, SY);
        StOtokov := StOtokov - 1;
      end; {if}
  end; {while StKopnih > 0}
  Assert(StOtokov = 1);
  
  { Izpiimo rezultat. }
  if ParamCount >= 2 then Assign(T, ParamStr(2)) else Assign(T, 'otoki.out');
  Rewrite(T); WriteLn(T, MaxStOtokov); Close(T);

{$IFDEF IZPISUJ}  
  WriteLn('Najve otokov: ', MaxStOtokov, ' pri gladini G = ', NajG);
{$ENDIF}
  
{$IFDEF QUICKSORT}
  FreeMem(VrstniRed, SizeOf(integer) * W * H);
{$ENDIF}
{$IFDEF HITRO_ZLIVANJE}
  FreeMem(VelOtoka, SizeOf(integer) * W * H);
  FreeMem(Vrsta, SizeOf(integer) * W * H);
{$ENDIF}
  
end. {Otoki}
