program Sbn; { RTK 2004.3.2 }

{ Urejanje z zlivanjem (merge sort).  Dobljeno zaporedje ukazov
  je sicer dolgo priblino 2n^2 ukazov (za urejanje n registrov),
  vendar pa traja njegovo izvajanje le O(n log n) korakov, ne pa
  O(n^2) kot pri urejanju z izbiranjem in urejanju z mehurki. 
  Manja slabost postopka je, da potrebuje n pomonih registrov. 
  
  Ideja urejanja z zlivanjem je, da lahko dve naraajoi zaporedji,
  dolgi po k elementov, brez teav zlijemo v eno samo naraajoe
  zaporedje z 2k elementi.  Prvotno neurejeno zaporedje n elementov
  si mislimo sestavljeno iz n naraajoih zaporedij s po enim elementom;
  ta podzaporedja (pravimo jim tudi "ete", angl. "runs") lahko zdaj
  po dve in dve zlivamo in dobimo n/2 naraajoih zaporedij s po dvema
  elementoma; zdaj zlivamo njih in dobimo n/4 naraajoih zaporedij
  s po tirimi elementi; itd. 
  
  Zlivanje zaporedij S1[0..m1-1] in S2[0..m2-1] v T[0..m1+m2-1]
  obiajno zapiemo nekako takole:
  
    i1 := 0; i2 := 0; 
    while (i1 < m1) or (i2 < m2) do begin
      -- Izberimo manjega od elementov S1[i1] in S2[i2].
      -- Izbrani element dodajmo v izhodno zaporedje T in 
      -- se premaknimo naprej po tistem vhodnem zaporedju, iz
      -- katerega je ta element priel.
      if S1[i1] < S2[i2] 
      then begin T[i1 + i2] := S1[i1]; i1 := i1 + 1 end 
      else begin T[i1 + i2] := S2[i2]; i2 := i2 + 1 end;
    end;
  
  (V gornji zanki manjka e posebna obravnava primerov i1 = m1 
  in i2 = m2.  e je i1 = m1, vrednost S1[i1] sploh ni definirana
  in moramo nujno vzeti element iz S2[i2] (lahko se delamo, da
  je S1[m1] = neskonno).  Analogen razmislek velja tudi pri i2 = m2.)
  
  Ker na namiljeni procesor z ukazi SBN ne podpira posrednega
  naslavljanja, bomo to zanko predelali tako, da bo imela po
  en izvod telesa zanke za vsak par (i1, i2), i1=0..m1, i2=0..m2:
  
    lab_i1_i2:   if S1[i1] < S2[i2] then begin
                   T[i1 + i2] := S1[i1];
                   goto lab_(i1+1)_i2;   -- to imamo namesto i1 := i1 + 1
                 end else begin
                   T[i1 + i2] := S2[i2];
                   goto lab_i1_(i2+1);   -- to imamo namesto i2 := i2 + 1
                 end;

  Poseben primer nastopi pri i2 = m2, ko je prisoten le prvi del
  tega stavka, in pri i1 = m1, ko je prisoten le drugi del. 

  Te kopije telesa zanke naj bodo zloene skupaj po naraajoih i1
  in pri vsakem i1 po naraajoih i2.  Tako skoka "goto lab_i1_(i2+1)"
  sploh ne bo treba zapisovati eksplicitno, ker bo prila ta labela
  tako ali tako tik za tem ukazom in se bo izvajanje e samo od sebe
  nadaljevalo pri njem.  Zato porabimo za vsak izvod telesa zanke
  najve tiri ukaze, za zlivanje dveh et doline k (v eno eto
  doline 2k) pa torej priblino 4k^2 ukazov.  Pri delu z n registri 
  imamo n/2 zlivanj et s po k=1 elementov, n/4 zlivanj et s po
  k=2 elementoma, ..., na koncu pa 1 zlivanje dveh et s po k = n/2 elementi.
  Skupna dolina kode bo torej priblino (naj bo n = 2^m)
    sum_(k=1,2,4,...,2^(m-1)) (n/(2k)) 4k^2
  = sum_(k=1,2,4,...,2^(m-1)) 2nk
  = 2n (2^m - 1) = pribl. 2n^2. }

var 
  T: text; n: integer;
  Stevec: integer; { za laje generiranje enolinih oznak (label) ukazov }

  { Zlije zaporedji r[s1..s1+m1-1] in r[s2..s2+m2-1] v r[d..m1+m2-1]. 
    Za to porabi O(m^2) intrukcij in O(m) asa. }
  procedure Zlivanje(s1, s2, d, m1, m2: integer);
  var i1, i2: integer; Lab: string;
  begin
    {WriteLn('zlivanje ', s1, '..', s1 + m1 - 1, ' in ', s2, '..', s2 + m2 - 1, ' v ', d, '..', d + m1 + m2 - 1);}
    Stevec := Stevec + 1; Str(Stevec, Lab);
    for i1 := 0 to m1 - 1 do for i2 := 0 to m2 do begin
      if (i1 > 0) or (i2 > 0) then Write(T, 'c_', Lab, '_', i1 * (m2 + 1) + i2, ': ');
      if i2 < m2 then 
        WriteLn(T, 'sbn r', 2 * n + 1, ', r', s2 + i2, ', r', s1 + i1, ', s_', Lab, '_', i1 * (m2 + 1) + i2);
      { r[s1 + i1] <= r[s2 + i2], premaknimo se torej po i1. }
      WriteLn(T, 'sbn r', d + i1 + i2, ', r', s1 + i1, ', 0, c_', Lab, '_', (i1 + 1) * (m2 + 1) + i2);
      WriteLn(T, 'sbn r', 2 * n + 1, ', 0, 1, c_', Lab, '_', (i1 + 1) * (m2 + 1) + i2);
      if i2 < m2 then begin
        Write(T, 's_', Lab, '_', i1 * (m2 + 1) + i2, ': ');
        { r[s2 + i2] < r[s1 + i1], premaknimo se torej po i2. }
        WriteLn(T, 'sbn r', d + i1 + i2, ', r', s2 + i2, ', 0');
      end; {if}
    end; {for i1, i2}
    { Obravnavajmo e primere, ko je i1 = m1.  Takrat le e kopiramo
      elemente iz r[s2+i2] v r[d+m1+i2] in poveujemo i2.  Ker si ti
      stavki sledijo drug za drugim po naraajoih i2, ni treba mednje
      dodajati nobenih eksplicitnih skokov. }
    for i2 := 0 to m2 - 1 do begin 
      Write(T, 'c_', Lab, '_', m1 * (m2 + 1) + i2, ': ');
      WriteLn(T, 'sbn r', d + m1 + i2, ', r', s2 + i2, ', 0');
    end; {for j}
    Write(T, 'c_', Lab, '_', m1 * (m2 + 1) + m2, ': ');
  end; {Zlivanje}

var Src, Dest, RunLen, s, d, m1, m2, Left: integer;
begin {Sbn}
  if ParamCount >= 1 then Assign(T, ParamStr(1)) else Assign(T, 'sbn.in');
  Reset(T); ReadLn(T, n); Close(T);
  if ParamCount >= 2 then Assign(T, ParamStr(2)) else Assign(T, 'sbn.out');
  Rewrite(T);
  
  Stevec := 0;
  RunLen := 1; Src := 1; Dest := n + 1;
  while RunLen < n do begin
    { Zdaj imamo v r[Src..Src+n-1] ete doline RunLen.
      Zlivali jih bomo v r[Dest..Dest+n-1]. }
    Left := n; s := Src; d := Dest;
    while Left > 1 do begin
      { Ostalo nam je e Left elementov, r[s..Src+n-1].  
        Vzemimo prvi dve eti med njimi -- naeloma ima vsaka
        po RunLen elementov, razen e ni ostalo manj kot 2 * RunLen
        elementov; v tem primeru je druga eta kraja ali pa imamo
        celo eno samo eto.  Dolini et oznaimo z m1 in m2. }
      m1 := Left; if m1 > RunLen then m1 := RunLen;
      m2 := Left - m1; if m2 > RunLen then m2 := RunLen;
      { Zlijmo obe eti.  Nova eta (doline m1 + m2) naj se vpie
        v ciljno skupino registrov od r[d] naprej. }
      Zlivanje(s, s + m1, d, m1, m2);
      Left := Left - m1 - m2;
      s := s + m1 + m2; d := d + m1 + m2;
    end; {while}
    if Left = 1 then
      WriteLn(T, 'sbn r', d, ', r', s, ', 0');
    { Zamenjajmo izvorno in ciljno skupino registrov.  To pomeni, da
      pri prvem izvajanju zunanje zanke zlivamo ete iz r[1..n] v
      r[n+1..2n], v drugem izvajanju ete iz r[n+1..2n] v r[1..n],
      v tretjem izvajanju spet iz r[1..n] v r[n+1..2n] in tako naprej. }
    s := Src; Src := Dest; Dest := s; 
    RunLen := 2 * RunLen; { ete so po zadnjem zlivanju postale dvakrat dalje. }
  end; {while}
  { Tole je le e en NOP, da bo na njem visela zadnja labela, ki jo
    je izpisal podprogram Zlivanje. }
  WriteLn(T, 'sbn r', 2 * n + 1, ', r', 2 * n + 1, ', r', 2 * n + 1);

  { Po zadnjem zlivanju nam je ostalo urejeno zaporedje v registrih
    r[Src..Src+n-1], naloga pa zahteva, da pripravimo urejeno zaporedje
    v registrih od [1..n]. }
  if Src <> 1 then
    for s := 1 to n do WriteLn(T, 'sbn r', Dest + s - 1, ', r', Src + s - 1, ', 0');
  Close(T);
end. {Sbn}
